/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.console.en;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.regex.Pattern;


/**
 * The class XConsole implements the interface IConsole, which
 * defines methods for the input and output of data of type String
 * (text), int (integral number) and float (floating point number)
 * in the context of simple console applications. An instance of
 * the class XConsole needs to be constructed with the new-operator
 * to get access to its non-static methods.
 * 
 * @author Frank Gollnik
 * @version 1.1.1
 */
public class XConsole implements IConsole
{
    // instance variables
    private String mPatternInteger;
    private String mPatternNumber;
    private BufferedReader mReader;

    /**
     * Public standard-constructor for objects of Type XConsole.
     */
    public XConsole()
    {
        // Initialization of instance variables
        // (see API documentation of java.util.regex.Pattern)
        mPatternInteger = "^[+-]?\\d+$"; // \d corresponds to [0-9]
        mPatternNumber = "^[-+]?[0-9]+[.]?[0-9]*([eE][-+]?[0-9]+)?$";
        mReader = new BufferedReader(new InputStreamReader(System.in));
    }

    public String readText()
    {
        String result = "";
        try
        {
            result = mReader.readLine();
            while (result.isEmpty())
            {
                System.out.println("The entered line is empty. Please repeat the input!");
                result = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return result;
    }

    public String readText(String prompt)
    {
        String result = "";
        try
        {
            System.out.println(prompt);
            result = mReader.readLine();
            while (result.isEmpty())
            {
                System.out.println("The entered line is empty. Please repeat the input!");
                result = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return result;
    }

    public int readInteger()
    {
        String line = "";
        try
        {
            line = mReader.readLine();
            while (!Pattern.matches(mPatternInteger, line))
            {
                System.out.println("The input does not have the format of an integer. Please repeat the input!");
                line = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Integer.parseInt(line);
    }

    public int readInteger(String prompt)
    {
        String line = "";
        try
        {
            System.out.println(prompt);
            line = mReader.readLine();
            while (!Pattern.matches(mPatternInteger, line))
            {
                System.out.println("The input does not have the format of an integer. Please repeat the input!");
                line = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Integer.parseInt(line);
    }

    public float readNumber()
    {
        String line = "";
        try
        {
            line = mReader.readLine();
            while (!Pattern.matches(mPatternNumber, line))
            {
                System.out.println("The input does not have the format of a number. Please repeat the input!");
                line = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Float.parseFloat(line);
    }

    public float readNumber(String prompt)
    {
        String line = "";
        try
        {
            System.out.println(prompt);
            line = mReader.readLine();
            while (!Pattern.matches(mPatternNumber, line))
            {
                System.out.println("The input does not have the format of a number. Please repeat the input!");
                line = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Float.parseFloat(line);
    }

    public void write(String text)
    {
        System.out.print(text);
    }

    public void write(int integer)
    {
        System.out.print(integer);
    }

    public void write(float number)
    {
        System.out.print(number);
    }

    public void writeLine(String text)
    {
        System.out.println(text);
    }

    public void writeLine(int integer)
    {
        System.out.println(integer);
    }

    public void writeLine(float number)
    {
        System.out.println(number);
    }
}
