/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


import org.jabotics.robot.de.*;


/**
 * Beispielprogramm für das Abspielen von Tönen und Klängen mit dem Lautsprecher.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public class B01_Jukebox extends XRoboter
{
    ////////////////////////////////////////////////////////////////////////////////////////
    // Hier werden die Objekte zur Robotersteuerung bereitgestellt.
    // Die Erzeugung der Objekte übernimmt für uns eine spezielle Klasse XObjektfabrik:
    ////////////////////////////////////////////////////////////////////////////////////////
    static final IMotor linkerMotor = objektfabrik.erzeugeMotor(EMotoranschluss.C);
    static final IMotor rechterMotor = objektfabrik.erzeugeMotor(EMotoranschluss.B);
    static final IPilot pilot = objektfabrik.erzeugePiloten(EMotoranschluss.C, EMotoranschluss.B);
    static final ITastsensor eingabetaste = objektfabrik.erzeugeTastsensor(ETaster.EINGABE);
    static final ITastsensor endetaste = objektfabrik.erzeugeTastsensor(ETaster.ENDE);
    static final ITastsensor linkstaste = objektfabrik.erzeugeTastsensor(ETaster.LINKS);
    static final ITastsensor rechtstaste = objektfabrik.erzeugeTastsensor(ETaster.RECHTS);
    static final ITastsensor tastsensor = objektfabrik.erzeugeTastsensor(ESensoranschluss.S1);
    static final ISensor schallsensor = objektfabrik.erzeugeSchallsensor(ESensoranschluss.S2);
    static final ILichtsensor lichtsensor = objektfabrik.erzeugeLichtsensor(ESensoranschluss.S3);
    static final ISensor ultraschallsensor = objektfabrik.erzeugeUltraschallsensor(ESensoranschluss.S4);
    static final IUhr uhr = objektfabrik.erzeugeUhr();
    static final IMonitor monitor = objektfabrik.erzeugeMonitor();
    static final ILautsprecher lautsprecher = objektfabrik.erzeugeLautsprecher();
 
     /**
     * Die Methode "main" ist der Startpunkt für das Roboterprogramm.
     * Diese Methode enthält die eigentliche Robotersteuerung, die du selbst programmieren sollst.
     * Ersetze das kleine Beispielprogramm durch deine eigenen Steuerungsbefehle.
     */
     public static void main(String[] args)
     {
        ////////////////////////////////////////////////////////////////////////////////////////
        // Beispiel-Robotersteuerung:
        ////////////////////////////////////////////////////////////////////////////////////////

        uhr.warteZeit(2000); // Wartezeit, bevor es richtig losgeht.

        // Das Motiv soll zweimal mithilfe von Tönen gespielt werden, deshalb alles in einer Schleife.
        // Die Lautstärke wird jedoch in beiden Durchläufen unterschiedlich sein.
        int n1 = 0;
        do
        {
            // 1. Takt
            n1 = n1 + 1;    // Zähler für Anzahl der Schleifendurchläufe wird um 1 erhöht.
            lautsprecher.setzeLautstaerke(100 - 30*n1); // 1. Durchlauf ist lauter als der erste
            
            lautsprecher.spieleTon(330,100);
            uhr.warteZeit(100);  // Pause von nochmals 100ms.
            lautsprecher.spieleTon(330,100);
            uhr.warteZeit(100);
            lautsprecher.spieleTon(330,100);
            lautsprecher.spielePause(100); // bewirkt das Gleiche wie  uhr.warteZeit(100)
            lautsprecher.spieleTon(440,1200);
            lautsprecher.spieleTon(660,600);

            // 2. und 3. Takt: wiederholt sich, deshalb in Schleife
            int n2 = 0;     
            do
            {
                n2 = n2 + 1;    // Zähler für Anzahl der Schleifendurchläufe wird um 1 erhöht.
                lautsprecher.spieleTon(587,190);
                lautsprecher.spielePause(10);
                lautsprecher.spieleTon(554,190);
                lautsprecher.spielePause(10);
                lautsprecher.spieleTon(494,190);
                lautsprecher.spielePause(10);
                lautsprecher.spieleTon(880,1200);
                lautsprecher.spieleTon(660,600);
            }
            while (n2 < 2);
            
            // 4. Takt
            lautsprecher.spieleTon(587,190);
            lautsprecher.spielePause(10);
            lautsprecher.spieleTon(554,190);
            lautsprecher.spielePause(10);
            lautsprecher.spieleTon(587,190);
            lautsprecher.spielePause(10);
            lautsprecher.spieleTon(494,1800);
        }
        while (n1 < 2);

        // Das Gleiche noch einmal, jetzt aber unter Verwendung der Methode lautsprecher.spieleKlang().
        lautsprecher.setzeLautstaerke(80);
        // Der Klang soll sich in beiden Durchläufen unterscheiden, deshalb Verwendung einer Variablen:
        EKlang klang;   // Deklaration einer neuen Variablen klang vom Datentyp EKlang
        n1 = 0; // Schleifenzähler wieder zurücksetzen! (Deklaration ist bereits vorher erfolgt.)
        do
        {
            // 1. Takt
            n1 = n1 + 1;    // Zähler für Anzahl der Schleifendurchläufe wird um 1 erhöht.
            if (n1 == 1)    // Achte auf den Unterschied zwischen "=" (Zuweisung) und "==" (Prüfung auf Gleichheit)
            {
                klang = KLAVIER;
            }
            else
            {
                klang = FLOETE;
            }

            lautsprecher.spieleKlang(klang,330,100);
            uhr.warteZeit(100);
            lautsprecher.spieleKlang(klang,330,100);
            uhr.warteZeit(100);
            lautsprecher.spieleKlang(klang,330,100);
            uhr.warteZeit(100);
            lautsprecher.spieleKlang(klang,440,1200);
            lautsprecher.spieleKlang(klang,660,600);

            // 2. und 3. Takt (wiederholt sich, deshalb in Schleife)
            // Diesmal muss die Variable n2 neu deklariert werden, denn die alte Variable n2
            // war nur innerhalb der ersten while-Schleife gültig.
            int n2 = 0;
            do
            {
                n2 = n2 + 1;    // Zähler für Anzahl der Schleifendurchläufe wird um 1 erhöht.
                lautsprecher.spieleKlang(klang,587,190);
                lautsprecher.spielePause(10);
                lautsprecher.spieleKlang(klang,554,190);
                lautsprecher.spielePause(10);
                lautsprecher.spieleKlang(klang,494,190);
                lautsprecher.spielePause(10);
                lautsprecher.spieleKlang(klang,880,1200);
                lautsprecher.spieleKlang(klang,660,600);
            }
            while (n2 < 2);
            
            // 4. Takt
            lautsprecher.spieleKlang(klang,587,190);
            lautsprecher.spielePause(10);
            lautsprecher.spieleKlang(klang,554,190);
            lautsprecher.spielePause(10);
            lautsprecher.spieleKlang(klang,587,190);
            lautsprecher.spielePause(10);
            lautsprecher.spieleKlang(klang,494,1800);
        }
        while (n1 < 2);

        // Schluss
        lautsprecher.setzeLautstaerke(100);
        lautsprecher.spieleKlang(XYLOPHON,330,190); // direkte Angabe des Klangs ohne Verwendung einer Variable
        lautsprecher.spielePause(10);
        lautsprecher.spieleKlang(XYLOPHON,330,190);
        lautsprecher.spielePause(10);
        lautsprecher.spieleKlang(XYLOPHON,330,190);
        lautsprecher.spielePause(10);
        lautsprecher.spieleKlang(XYLOPHON,440,600);
        lautsprecher.spieleKlang(XYLOPHON,330,600);
        lautsprecher.spieleKlang(XYLOPHON,440,600);
        lautsprecher.spieleKlang(XYLOPHON,330,600);
        lautsprecher.spieleKlang(XYLOPHON,220,2400);
    }
}
