/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


import org.jabotics.robot.de.*;


/**
 * Beispielprogramm für die Nutzung der einzelnen Motoren zur Bewegungssteuerung.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public class B03_Tanzmarie extends XRoboter
{
    ////////////////////////////////////////////////////////////////////////////////////////
    // Hier werden die Objekte zur Robotersteuerung bereitgestellt.
    // Die Erzeugung der Objekte übernimmt für uns eine spezielle Klasse XObjektfabrik:
    ////////////////////////////////////////////////////////////////////////////////////////
    static final IMotor linkerMotor = objektfabrik.erzeugeMotor(EMotoranschluss.C);
    static final IMotor rechterMotor = objektfabrik.erzeugeMotor(EMotoranschluss.B);
    static final IPilot pilot = objektfabrik.erzeugePiloten(EMotoranschluss.C, EMotoranschluss.B);
    static final ITastsensor eingabetaste = objektfabrik.erzeugeTastsensor(ETaster.EINGABE);
    static final ITastsensor endetaste = objektfabrik.erzeugeTastsensor(ETaster.ENDE);
    static final ITastsensor linkstaste = objektfabrik.erzeugeTastsensor(ETaster.LINKS);
    static final ITastsensor rechtstaste = objektfabrik.erzeugeTastsensor(ETaster.RECHTS);
    static final ITastsensor tastsensor = objektfabrik.erzeugeTastsensor(ESensoranschluss.S1);
    static final ISensor schallsensor = objektfabrik.erzeugeSchallsensor(ESensoranschluss.S2);
    static final ILichtsensor lichtsensor = objektfabrik.erzeugeLichtsensor(ESensoranschluss.S3);
    static final ISensor ultraschallsensor = objektfabrik.erzeugeUltraschallsensor(ESensoranschluss.S4);
    static final IUhr uhr = objektfabrik.erzeugeUhr();
    static final IMonitor monitor = objektfabrik.erzeugeMonitor();
    static final ILautsprecher lautsprecher = objektfabrik.erzeugeLautsprecher();
 
     /**
     * Die Methode "main" ist der Startpunkt für das Roboterprogramm.
     * Diese Methode enthält die eigentliche Robotersteuerung, die du selbst programmieren sollst.
     * Ersetze das kleine Beispielprogramm durch deine eigenen Steuerungsbefehle.
     */
    public static void main(String[] args)
    {
        ////////////////////////////////////////////////////////////////////////////////////////
        // Beispiel-Robotersteuerung:
        ////////////////////////////////////////////////////////////////////////////////////////

        uhr.warteZeit(2000); // Zeit, um die Hand nach dem Programmstart aus dem Weg zu nehmen
        
        // In diesem Beispiel werden die Motoren einzeln angesteuert.
        linkerMotor.setzeSollgeschwindigkeit(720);  // in Grad/s
        linkerMotor.setzeSollbeschleunigung(3000);  // in Grad/s²
        rechterMotor.setzeSollgeschwindigkeit(720);
        rechterMotor.setzeSollbeschleunigung(3000);

        int n = 0;
        do
        {
            n = n + 1;  // Schleifenzähler erhöhen

            // 1.Takt
            rechterMotor.dreheZeit(500,VORWAERTS,STARTEN);    // Vorwärtsdrehung für 500ms
            lautsprecher.spieleTon(440,250);
            rechterMotor.warteEndeAb();

            rechterMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
            lautsprecher.spieleTon(660,250);
            rechterMotor.warteEndeAb();

            linkerMotor.dreheZeit(500,VORWAERTS,STARTEN);
            rechterMotor.dreheZeit(500,VORWAERTS,STARTEN);
            lautsprecher.spieleTon(330,250);
            rechterMotor.warteEndeAb();

            linkerMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
            rechterMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
            lautsprecher.spieleTon(660,250);
            rechterMotor.warteEndeAb();

            // 2.Takt
            linkerMotor.dreheZeit(500,VORWAERTS,STARTEN);
            lautsprecher.spieleTon(494,250);
            linkerMotor.warteEndeAb();

            linkerMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
            lautsprecher.spieleTon(660,250);
            linkerMotor.warteEndeAb();

            linkerMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
            rechterMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
            lautsprecher.spieleTon(330,250);
            rechterMotor.warteEndeAb();

            linkerMotor.dreheZeit(500,VORWAERTS,STARTEN);
            rechterMotor.dreheZeit(500,VORWAERTS,STARTEN);
            lautsprecher.spieleTon(660,250);
            rechterMotor.warteEndeAb();

            // 3.Takt
            linkerMotor.dreheZeit(1000,RUECKWAERTS,STARTEN);
            rechterMotor.dreheZeit(1000,VORWAERTS,STARTEN);
            lautsprecher.spieleTon(440,1000);

            linkerMotor.dreheZeit(1000,VORWAERTS,STARTEN);
            rechterMotor.dreheZeit(1000,RUECKWAERTS,STARTEN);
            lautsprecher.spieleTon(554,1000);

            // 4.Takt
            linkerMotor.dreheZeit(1000,VORWAERTS,STARTEN);
            rechterMotor.dreheZeit(1000,RUECKWAERTS,STARTEN);
            lautsprecher.spieleTon(660,1000);

            linkerMotor.dreheZeit(1000,RUECKWAERTS,STARTEN);
            rechterMotor.dreheZeit(1000,VORWAERTS,STARTEN);
            lautsprecher.spieleTon(415,1000);
        }
        while (n < 4);

        linkerMotor.dreheZeit(500,VORWAERTS,STARTEN);
        rechterMotor.dreheZeit(500,VORWAERTS,STARTEN);
        lautsprecher.spieleTon(440,500);

        linkerMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
        rechterMotor.dreheZeit(500,RUECKWAERTS,STARTEN);
        lautsprecher.spieleTon(330,500);

        linkerMotor.dreheZeit(1000,VORWAERTS,STARTEN);
        rechterMotor.dreheZeit(1000,VORWAERTS,STARTEN);
        lautsprecher.spieleTon(440,1000);

    }
}
