/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> Die Schnittstelle
 * IMonitor bietet Methoden zur Ausgabe auf dem Display des Roboter-Bausteins.
 * Bei Angaben von Positionen auf dem Monitor ist zu beachten, dass die Nummerierungen von
 * Zeilen (y-Position) und von x-Positionen innerhalb einer Zeile jeweils bei 0 beginnen.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */

public interface IMonitor
{
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Führt zu einer Aktualisierung der Anzeige.
     */
    public void aktualisiereAnzeige();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Löscht den gesamten Inhalt des Monitors.
     */
    public void loescheAlles();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Löscht eine einzelne Textzeile auf dem Monitor.
     * 
     * @param  y_position   die Zeilennummer der zu löschenden Textzeile
     */
    public void loescheZeile(int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Löscht eine Folge von Zeichen auf dem Monitor.
     * 
     * @param  x_position   x-Position des ersten zu löschenden Zeichens
     * @param  y_position   die Zeilennummer der zu löschenden Zeichen
     * @param  anzahl       Anzahl der zu löschenden Zeichen
     */
    public void loescheZeichen(int x_position, int y_position, int anzahl);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Schreibt ein einzelnes Zeichen auf den Monitor.
     * 
     * @param  zeichen      das zu schreibende Zeichen (keine Umlaute)
     * @param  x_position   x-Position des zu schreibenden Zeichens
     * @param  y_position   die Zeilennummer des zu schreibenden Zeichens
     */
    public void schreibe(char zeichen, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Schreibt eine ganze Zahl auf den Monitor.
     * 
     * @param  ganzzahl     die zu schreibende Ganzzahl
     * @param  x_position   x-Position des Anfangs der zu schreibenden Zahl
     * @param  y_position   die Zeilennummer der zu schreibenden Zahl
     */
    public void schreibe(int ganzzahl, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Schreibt eine ganze Zahl auf den Monitor.
     * 
     * @param  ganzzahl         die zu schreibende Ganzzahl
     * @param  x_position       x-Position des Anfangs der zu schreibenden Zahl
     * @param  y_position       die Zeilennummer der zu schreibenden Zahl
     * @param  anzahl_stellen   Stellenzahl, mit der die Zahl dargestellt wird (links wird aufgefüllt)
     */
    public void schreibe(int ganzzahl, int x_position, int y_position, int anzahl_stellen);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Schreibt eine Fließkommazahl auf den Monitor.
     * 
     * @param  fliesskommazahl  die zu schreibende Fließkommazahl
     * @param  x_position       x-Position des Anfangs der zu schreibenden Zahl
     * @param  y_position       die Zeilennummer der zu schreibenden Zahl
     */
    public void schreibe(float fliesskommazahl, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Schreibt eine Zeichenkette auf den Monitor.
     * 
     * @param  text         die zu schreibende Zeichenkette
     * @param  x_position   x-Position des Anfangs der zu schreibenden Zeichenkette
     * @param  y_position   die Zeilennummer der zu schreibenden Zeichenkette
     */
    public void schreibe(String text, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Schreibt eine Zeichenkette auf den Monitor.
     * 
     * @param  text           die zu schreibende Zeichenkette
     * @param  x_position     x-Position des Anfangs der zu schreibenden Zeichenkette
     * @param  y_position     die Zeilennummer der zu schreibenden Zeichenkette
     * @param  hervorgehoben  bei Übergabe des Wertes "true" wird die Zeichenkette hervorgehoben dargestellt
     */
    public void schreibe(String text, int x_position, int y_position, boolean hervorgehoben);
}
