/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;


/**
 * <font color="Gold"><b>&bull;</b></font> Die Schnittstelle IObjektfabrik dient der Erzeugung
 * aller Objekte zur Steuerung eines Roboters (wie Sensor- und Motorobjekte)
 * und liefert die zugehörigen Schnittstellen auf diese Objekte zurück.
 * Anfänger werden diese Schnittstelle vermutlich nicht selbst nutzen. Stattdessen kann man
 * eine Vorlage bereitstellen, in der die Erzeugung aller benötigten Steuerungsobjekte
 * enthalten ist. Während Anfänger z.B. direkt in der main-Methode einer Roboterklasse
 * die Befehle für ihre eigene Robotersteuerung einfügen, kann eine andere Vorlage für
 * Fortgeschrittene die Verwendung von Instanzvariablen und weiteren Klassenmethoden vorsehen.
 * Verwendet man beispielsweise BlueJ als Entwicklungsumgebung, so kann man Vorlagen
 * (sogenannte Templates) einfach in dem dafür vorgesehenen Template-Ordner hinterlegen.
 * Bei Erzeugung einer neuen Klasse wählt der Anwender dann eine Vorlage
 * aus den verfügbaren Klassentemplates aus.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */

public interface IObjektfabrik
{
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Motor repräsentiert.
     * 
     * @param  anschluss  Anschluss, über den der Motor verbunden ist
     * @return            IMotor-Schnittstelle auf das neue Motor-Objekt
     */
    public IMotor erzeugeMotor(EMotoranschluss anschluss);
    
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Autopiloten zur Steuerung beider Motor repräsentiert.
     * Es werden Standardwerte für den Raddurchmesser sowie den Radabstand angenommen.
     * 
     * @param  anschlussLinkerMotor    Anschluss, über den der linke Motor verbunden ist
     * @param  anschlussRechterMotor   Anschluss, über den der rechte Motor verbunden ist
     * @return                         IPilot-Schnittstelle auf das neue Pilot-Objekt
     */
    public IPilot erzeugePiloten(EMotoranschluss anschlussLinkerMotor, EMotoranschluss anschlussRechterMotor);
    
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Autopiloten zur Steuerung beider Motor repräsentiert.
     * 
     * @param  anschlussLinkerMotor    Anschluss, über den der linke Motor verbunden ist
     * @param  anschlussRechterMotor   Anschluss, über den der rechte Motor verbunden ist
     * @param  raddurchmesser          Durchmesser eines der Räder an den Motoren (standardmäßig in cm)
     * @param  radabstand              Radabstand von linkem und rechtem Motor (standardmäßig in cm)
     * @return                         IPilot-Schnittstelle auf das neue Pilot-Objekt
     */
    public IPilot erzeugePiloten(EMotoranschluss anschlussLinkerMotor, EMotoranschluss anschlussRechterMotor, float raddurchmesser, float radabstand);

    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Drucktaster des Roboter-Bausteins repräsentiert.
     * 
     * @param  taster  Typ des Tasters (EINGABE, ENDE, LINKS, RECHTS)
     * @return         ITastsensor-Schnittstelle auf das neue Taster-Objekt
     */
    public ITastsensor erzeugeTastsensor(ETaster taster);

    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Tastsensor repräsentiert.
     * 
     * @param  anschluss  Anschluss, über den der Sensor verbunden ist
     * @return            ITastsensor-Schnittstelle auf das neue Tastsensor-Objekt
     */
    public ITastsensor erzeugeTastsensor(ESensoranschluss anschluss);
    
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Schallsensor zur Lautstärkemessung repräsentiert.
     * 
     * @param  anschluss  Anschluss, über den der Sensor verbunden ist
     * @return            ISensor-Schnittstelle auf das neue Schallsensor-Objekt
     */
    public ISensor erzeugeSchallsensor(ESensoranschluss anschluss);
    
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Lichtsensor zur Helligkeitsmessung repräsentiert.
     * 
     * @param  anschluss  Anschluss, über den der Sensor verbunden ist
     * @return            ILichtsensor-Schnittstelle auf das neue Lichtsensor-Objekt
     */
    public ILichtsensor erzeugeLichtsensor(ESensoranschluss anschluss);
    
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die einen Ultraschallsensor zur Abstandsmessung repräsentiert.
     * 
     * @param  anschluss  Anschluss, über den der Sensor verbunden ist
     * @return            ISensor-Schnittstelle auf das neue Ultraschallsensor-Objekt
     */
    public ISensor erzeugeUltraschallsensor(ESensoranschluss anschluss);
    
    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die den LCD-Monitor des Roboter-Bausteins repräsentiert.
     * 
     * @return        IMonitor-Schnittstelle auf das neue Monitor-Objekt
     */
    public IMonitor erzeugeMonitor();

    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die den Lautsprecher des Roboter-Bausteins repräsentiert.
     * 
     * @return        ILautsprecher-Schnittstelle auf das neue Lautsprecher-Objekt
     */
    public ILautsprecher erzeugeLautsprecher();

    /**
     * <font color="Gold"><b>&bull;</b></font> Erzeugt eine Objektinstanz für die Zeitmessung.
     * 
     * @return        IUhr-Schnittstelle auf das neue Zeitmesser-Objekt
     */
    public IUhr erzeugeUhr();

    /**
     * <font color="Red"><b>&bull;</b></font> Erzeugt eine Objektinstanz, die eine Anzeige-Konsole auf einem verbundenen PC repräsentiert.
     * 
     * @return        IPCKonsole-Schnittstelle auf das neue PCKonsole-Objekt
     */
    public IPCKonsole erzeugePCKonsole();
}
