/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> Die Schnittstelle ISensor bietet Methoden zur Steuerung und zum Auslesen von Sensoren.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */

public interface ISensor
{
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Liefert einen einzelnen Messwert des Sensors. 
     * Die möglichen Messwerte hängen vom Sensortyp ab:
     * Werte 0 (nicht gedrückt) und 1 (gedrückt) für Drucksensoren,
     * Wertebereich 0 (leise) bis 100 (laut) für Schallsensoren,
     * Wertebereich 0 (dunkel) bis 100 (hell) für Lichtsensoren,
     * der Abstand zum nächsten Hindernis in cm für Ultraschallsensoren.
     * Dabei ist zu beachten, dass die maximale Reichweite des Ultraschallsensors
     * begrenzt ist und typischweise nur etwa 250 cm beträgt.
     * 
     * @return      das Ergebnis der Messung
     */
    public float holeMesswert();

    /**
     * <font color="Gold"><b>&bull;</b></font> Liefert die zeitliche Änderungsrate der Messwerte.
     * Die Sensoren überwachen ständig die Veränderung der Messgröße und ermitteln die zeitliche
     * Änderungsrate der Messwerte.
     * 
     * @return  die zeitlich Änderungsrate der Messgröße in Einheiten pro Sekunde
     */
    public float holeAenderungsrate();

    /**
     * <font color="Red"><b>&bull;</b></font> Liefert die aktuelle Toleranzeinstellung
     * für die Filterung nach Messwerten bei der Ereignisverarbeitung von Sensordaten.
     * Die Messwerttoleranz gibt an, ab welcher absoluten Änderung des Messwerts des Sensors
     * eine Änderungsmeldung an einen angemeldeten Sensorbeobachter verschickt wird
     * (siehe {@link #setzeBeobachter(ISensorbeobachter)} und
     * {@link ISensorbeobachter#registriereEreignis(ISensor,float,float,float,float)}).
     * Standardmäßig ist für die Toleranz der Messwerte der Wert 1 eingestellt.
     * Für die meisten Sensoren entspricht dies der tatsächlichen Auflösung der Sensorrohdaten.
     * Der sinnvolle Toleranzbereich wird nach oben durch den Messwertebereich eines Sensors begrenzt
     * (siehe {@link #holeMesswert()}). Ist der Toleranzwert für Messwerte zu groß eingegestellt,
     * werden keine Sensorereignisse gemeldet. Ein Toleranzwert von 0 deaktiviert die Ereignisfilterung,
     * so dass jeder neue Messwert an den Beobachter gemeldet wird. Neben der Filterung nach Messwerten
     * ist allerdings auch die Filterung nach der Änderungsrate zu beachten
     * (siehe {@link #holeFiltertoleranzAenderungsrate()}).
     * Der Tastsensor bildet bei der Filterung nach Messwerten einen Sonderfall, da dieser
     * nur die möglichen Messwerte 0 und 1 liefert. Hier besitzt die Änderungstoleranz immer
     * den Wert 1 und ist nicht veränderbar.
     * 
     * @return      die aktuelle Toleranzeinstellung für Messwerte bei der Ereignisfilterung von Sensordaten
     * @see         #holeFiltertoleranzAenderungsrate()
     * @see         #setzeFiltertoleranzen(float,float)
     * @see         #setzeBeobachter(ISensorbeobachter)
     * @see         #loescheBeobachter()
     * @see         ISensorbeobachter
     */
    public float holeFiltertoleranzMesswert();

    /**
     * <font color="Red"><b>&bull;</b></font> Liefert die aktuelle Toleranzeinstellung
     * für die Filterung nach Änderungsraten bei der Ereignisverarbeitung von Sensordaten.
     * Der Toleranzwert gibt an, ab welcher absoluten Änderung der Änderungsrate für den
     * Messwert eines Sensors eine Änderungsmeldung an einen angemeldeten Sensorbeobachter
     * verschickt wird (siehe {@link #setzeBeobachter(ISensorbeobachter)} und
     * {@link ISensorbeobachter#registriereEreignis(ISensor,float,float,float,float)}).
     * Standardmäßig ist für die Toleranz der Änderungsrate der Wert 0 eingestellt,
     * d.h. die Filterung nach der Änderungsrate ist deaktiviert. Ist ein Wert größer
     * als 0 eingestellt, werden neue Messdaten nur dann an einen angemeldeten Beobachter
     * geschickt, wenn sich die Änderungsrate seit der letzten Ereignismeldung um
     * mindestens den Toleranzwert verändert hat. Neben der Filterung nach der Änderungsrate
     * ist allerdings auch die Filterung nach Messwerten zu beachten
     * (siehe {@link #holeFiltertoleranzMesswert()}).
     * 
     * @return      die aktuelle Toleranzeinstellung für die Änderungsrate bei der Ereignisfilterung von Sensordaten
     * @see         #holeFiltertoleranzMesswert()
     * @see         #setzeFiltertoleranzen(float,float)
     * @see         #setzeBeobachter(ISensorbeobachter)
     * @see         #loescheBeobachter()
     * @see         ISensorbeobachter
     */
    public float holeFiltertoleranzAenderungsrate();
    
    /**

     * <font color="Gold"><b>&bull;</b></font> Liefert die Zeiteinstellung für die Trägheit des Sensors.
     * Sensoren führen intern laufend Messungen der Messgröße durch. Die dabei registrierten Werte
     * werden über einen gewissen Zeitraum gemittelt. Wenn man mit {@link #holeMesswert()}
     * einen Messwert beim Sensor abfragt, erhält man diesen Mittelwert. Die Trägheitseinstellung
     * bestimmt, über welchen Zeitraum (in ms) die intern ermittelten Messwerte gemittelt werden,
     * um den Ausgabewert zu berechnen. Ein großer Mittelungszeitraum entspricht einer
     * großen Sensorträgheit. Die Ausgangseinstellung für die Trägheit ist 0 (ms), d.h. es wird
     * keine Zeitmittelung der Messwerte durchgeführt. Der Wert der Trägheit beeinflusst auch
     * die ermittelte Änderungsrate für den Sensor (siehe {@link #holeAenderungsrate()}).
     * 
     * @return        die aktuelle Zeiteinstellung für die Trägheit des Sensors in ms
     */
    public int holeTraegheit();
    
    /**
     * <font color="Red"><b>&bull;</b></font> Setzt die Toleranzen für die Ereignisfilterung von Sensordaten.
     * 
     * @param       toleranz_messwert        die neue Toleranz für die Ereignisfilterung nach Messwerten
     * @param       toleranz_aenderungsrate  die neue Toleranz für die Ereignisfilterung nach der Änderungsrate
     * @see         #holeFiltertoleranzMesswert()
     * @see         #holeFiltertoleranzAenderungsrate()
     * @see         #setzeBeobachter(ISensorbeobachter)
     * @see         #loescheBeobachter()
     * @see         ISensorbeobachter
     */
    public void setzeFiltertoleranzen(float toleranz_messwert, float toleranz_aenderungsrate);

    /**
     * <font color="Gold"><b>&bull;</b></font> Setzt die Trägheitseinstellung des Sensors.
     * 
     * @param  zeit    die neue Zeitdauer für die Messwertmittelung in ms
     * @see    #holeTraegheit()
     */
    public void setzeTraegheit(int zeit);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung, bis der vorgegebene Grenzwert überschritten wird.
     * 
     * @param grenzwert     der Grenzwert, der überschritten werden muss, damit die Methode zurückkehrt
     */
    public void warteBisGrenzwertUeberschritten(float grenzwert);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung, bis der vorgegebene Grenzwert unterschritten wird.
     * 
     * @param grenzwert     der Grenzwert, der unterschritten werden muss, damit die Methode zurückkehrt
     */
    public void warteBisGrenzwertUnterschritten(float grenzwert);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung, bis der vorgegebene Grenzwert oder die maximale Wartezeit überschritten wird.
     * 
     * @param grenzwert     der Grenzwert, der überschritten werden muss, damit die Methode vorzeitig zurückkehrt
     * @param maximale_wartezeit     die maximale Wartezeit, in der auf die Überschreitung des Grenzwerts gewartet wird
     * 
     * @return        true, falls Grenzwert überschritten bzw. false, falls maximale Wartezeit überschritten wurde
     */
    public boolean warteBisGrenzwertUeberschritten(float grenzwert, int maximale_wartezeit);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung, bis der vorgegebene Grenzwert unterschritten oder die maximale Wartezeit überschritten wird.
     * 
     * @param grenzwert     der Grenzwert, der unterschritten werden muss, damit die Methode vorzeitig zurückkehrt
     * @param maximale_wartezeit     die maximale Wartezeit, in der auf die Unterschreitung des Grenzwerts gewartet wird
     * 
     * @return        true, falls Grenzwert unterschritten bzw. false, falls maximale Wartezeit überschritten wurde
     */
    public boolean warteBisGrenzwertUnterschritten(float grenzwert, int maximale_wartezeit);

    /**
     * <font color="Red"><b>&bull;</b></font> Anmelden eines Sensorbeobachters.
     * Ein vorher gesetzter Sensorbeobachter wird durch den neuen Sensorbeobachter ersetzt.
     * Während des Methodenaufrufs werden noch alle gegebenenfalls bestehenden Ereignismeldungen
     * an den vorherigen Beobachter abgearbeitet, und eine initiale Ereignismeldung wird an den
     * neuen Beobachter verschickt. Die Ausführung dieser Methode benötigt wegen der notwendigen
     * Synchronisation der beteiligten Hintergrundprozesse etwas Zeit (typischerweise 10 ms oder mehr).
     * 
     * @param  beobachter   der Sensorbeobachter, der die Ereignismeldungen erhalten soll
     * 
     * @see    #setzeFiltertoleranzen(float,float)
     * @see    #loescheBeobachter()
     */
    public void setzeBeobachter(ISensorbeobachter beobachter);
    
    /**
     * <font color="Red"><b>&bull;</b></font> Abmelden des registrierten Sensorbeobachters.
     * Es gelten entsprechende Bedingungen wie für {@link #setzeBeobachter(ISensorbeobachter)}.
     * 
     * @see    #setzeBeobachter(ISensorbeobachter)
     */
    public void loescheBeobachter();
    
    /**
     * <font color="Red"><b>&bull;</b></font> Überprüfung auf Überlauf bei der Ereignisverarbeitung.
     * Durch den Aufruf dieser Methode wird der Überlaufstatus wieder zurückgesetzt.
     * 
     * @return  liefert false bei erfolgreicher Ereignisverarbeitung und true bei Überlauf
     * @see     ISensorbeobachter
     */
    public boolean pruefeAufEreignisueberlauf();
}
