/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.core.CPilot;

/**
 * Objekte vom Typ XPilot repräsentieren einen Autopiloten für die Bewegungssteuerung
 * eines NXT-Roboters im Standardaufbau mit je einem Motoren links und rechts.
 * Die Klasse XPilot stellt dem Nutzer die Schnittstelle {@link IPilot} zur Verfügung.
 * Der Konstruktor der Klasse ist "nicht öffentlich". Man kann deshalb
 * als Anwender selbst keine Objekte dieses Typs erzeugen.
 * Man verschafft sich Zugang zu einem neuen Pilot-Objekt, indem man die Methode
 * {@link IObjektfabrik#erzeugePiloten(EMotoranschluss linkerMotor, EMotoranschluss rechterMotor, float raddurchmesser, float radabstand)}
 * aufruft.
 * Man erhält Zugriff auf die einzige Objektfabrik-Instanz, indem man
 * die statische Methode {@link XObjektfabrik#holeObjektfabrik()} aufruft.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
class XPilot implements IPilot
{
    
    // Instanzvariablen - ersetzen Sie das folgende Beispiel mit Ihren Variablen
    private CPilot mPilot;

    /**
     * Konstruktor für Objekte der Klasse XPilot
     * Es werden Standardwerte für den Raddurchmesser und den Radabstand angenommen.
     */
    XPilot(EMotoranschluss anschlussLinkerMotor, EMotoranschluss anschlussRechterMotor)
    {
        mPilot = CPilot.getPilotSingleton(EMotorPort.values()[anschlussLinkerMotor.ordinal()],EMotorPort.values()[anschlussRechterMotor.ordinal()]);
    }

    /**
     * Konstruktor für Objekte der Klasse XPilot
     */
    XPilot(EMotoranschluss anschlussLinkerMotor, EMotoranschluss anschlussRechterMotor, float raddurchmesser, float radabstand)
    {
        mPilot = CPilot.getPilotSingleton(EMotorPort.values()[anschlussLinkerMotor.ordinal()],EMotorPort.values()[anschlussRechterMotor.ordinal()],raddurchmesser, radabstand);
    }

    @Override
    public void warteEndeAb()
    {
        mPilot.waitUntilCompleted();
    }
    
    @Override
    public void halteAn(EWartemodus wartemodus)
    {
        mPilot.stopMotion(EWartemodus.ABWARTEN == wartemodus);
    }

    @Override
    public float holeRaddurchmesser()
    {
        return mPilot.getWheelDiameter();
    }

    @Override
    public void setzeRaddurchmesser(float durchmesser)
    {
        mPilot.setWheelDiameter(durchmesser);
    }

    @Override
    public float holeRadabstand()
    {
        return mPilot.getWheelbase();
    }

    @Override
    public void setzeRadabstand(float abstand)
    {
        mPilot.setWheelbase(abstand);
    }

    @Override
    public float holeGeschwindigkeit()
    {
        return mPilot.getSpeed();
    }

    @Override
    public float holeSollgeschwindigkeit()
    {
        return mPilot.getTargetSpeed();
    }

    @Override
    public void setzeSollgeschwindigkeit(float geschwindigkeit)
    {
        mPilot.setTargetSpeed(geschwindigkeit);
    }

    @Override
    public float holeMaximalgeschwindigkeit()
    {
        return mPilot.getMaxSpeed();
    }

    @Override
    public void setzeSollgeschwindigkeitMaximal()
    {
        mPilot.setTargetSpeedToMax();
    }

    @Override
    public float holeSollbeschleunigung()
    {
        return mPilot.getTargetAcceleration();
    }

    @Override
    public void setzeSollbeschleunigung(float beschleunigung)
    {
        mPilot.setTargetAcceleration(beschleunigung);
    }

    @Override
    public float holeMaximalbeschleunigung()
    {
        return mPilot.getMaxAcceleration();
    }

    @Override
    public void setzeSollbeschleunigungMaximal()
    {
        mPilot.setTargetAccelerationToMax();
    }

    @Override
    public float holeTachostand()
    {
        return mPilot.getTachoCount();
    }
    
    @Override
    public float holeAusrichtungswinkel()
    {
        return mPilot.getOrientation();
    }

    @Override
    public void setzeZaehlerstaendeZurueck()
    {
        mPilot.resetCounters();
    }

    @Override
    public void fahreGerade(ERichtung richtung)
    {
        mPilot.moveStraight(ERichtung.VORWAERTS == richtung);
    }

    @Override
    public void fahreGerade(float strecke, EWartemodus wartemodus)
    {
        mPilot.moveStraight(strecke, EWartemodus.ABWARTEN == wartemodus);
    }

    @Override
    public void fahreLenkung(float lenkung, ERichtung richtung)
    {
        mPilot.moveCurvature(lenkung, ERichtung.VORWAERTS == richtung);
    }

    @Override
    public void fahreRadius(float radius, ERichtung richtung)
    {
        mPilot.moveRadius(radius, ERichtung.VORWAERTS == richtung);
    }

    @Override
    public void fahreRadius(float radius, float drehwinkel, EWartemodus wartemodus)
    {
        mPilot.moveArc(D.DEGREE_TO_RAD_F * radius * drehwinkel, drehwinkel, EWartemodus.ABWARTEN == wartemodus);
    }

    @Override
    public void fahreBogen(float strecke, float drehwinkel, EWartemodus wartemodus)
    {
        mPilot.moveArc(strecke, drehwinkel, EWartemodus.ABWARTEN == wartemodus);
    }

    @Override
    public void dreheUmWinkel(float drehwinkel, EWartemodus wartemodus)
    {
        mPilot.rotateByAngle(drehwinkel, EWartemodus.ABWARTEN == wartemodus);
    }

    @Override
    public void dreheZuAusrichtungswinkel(float ausrichtungswinkel, EWartemodus wartemodus)
    {
        mPilot.rotateToOrientation(ausrichtungswinkel, EWartemodus.ABWARTEN == wartemodus);
    }

    @Override
    public boolean istInBewegung()
    {
        return mPilot.isMoving();
    }
}
