/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

import org.jabotics.robot.defs.ESensorType;
import org.jabotics.robot.defs.ESensorPort;
import org.jabotics.robot.defs.EButtonType;
import org.jabotics.robot.core.CSensor;
import org.jabotics.robot.core.CEventDispatcher;

/**
 * Die Basisklasse XSensor stellt die Basisfunktionalitäten für Sensoren bereit.
 * Ableitungen ergänzen die Funktionaltät für speziellere Sensorklassen.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
class XSensor implements ISensor
{
    // Instanzvariablen
    protected CSensor mSensor; // wird in Konstruktur der abgeleiteten Klassen gesetzt

    private volatile EventAdapter mAdapter = new EventAdapter();
    private volatile EventAdapter mNewAdapter = new EventAdapter();

    private class EventAdapter implements CEventDispatcher.IHandler
    {
        private volatile ISensorbeobachter mBeobachter = null;

        public void manageEvent(int port, float oldValue, float newValue, float oldRate, float newRate)
        {
            mBeobachter.registriereEreignis(XSensor.this,oldValue,newValue,oldRate,newRate);
        }
    }

    XSensor(ESensoranschluss anschluss, ESensorType type)
    {
        mSensor = CSensor.getSensor(ESensorPort.values()[anschluss.ordinal()], type);
    }

    XSensor(EButtonType type)
    {
        mSensor = CSensor.getButton(type);
    }
    
    public float holeMesswert()
    {
        return mSensor.getOutputValue();
    }

    public float holeAenderungsrate()
    {
        return mSensor.getChangeRate();
    }

    public float holeFiltertoleranzMesswert()
    {
        return mSensor.getEventFilterValue();
    }

    public float holeFiltertoleranzAenderungsrate()
    {
        return mSensor.getEventFilterRate();
    }
    
    public int holeTraegheit()
    {
        return mSensor.getIntegrationTime();
    }

    public void setzeFiltertoleranzen(float aufloesung_messwert, float aufloesung_aenderungsrate)
    {
        mSensor.setEventFilter(aufloesung_messwert, aufloesung_aenderungsrate);
    }

    public void setzeTraegheit(int zeit)
    {
        mSensor.setIntegrationTime(zeit);
    }

    public void warteBisGrenzwertUeberschritten(float grenzwert)
    {
        mSensor.waitUntilBeyondLimit(grenzwert);
    }

    public void warteBisGrenzwertUnterschritten(float grenzwert)
    {
        mSensor.waitUntilBelowLimit(grenzwert);
    }

    public boolean warteBisGrenzwertUeberschritten(float grenzwert, int maximale_wartezeit)
    {
        return mSensor.waitUntilBeyondLimit(grenzwert, maximale_wartezeit);
    }

    public boolean warteBisGrenzwertUnterschritten(float grenzwert, int maximale_wartezeit)
    {
        return mSensor.waitUntilBelowLimit(grenzwert, maximale_wartezeit);
    }

    public synchronized void setzeBeobachter(ISensorbeobachter beobachter)
    {
        if (beobachter != mAdapter.mBeobachter)
        {
            mNewAdapter.mBeobachter = beobachter;
            mSensor.setEventHandler( (null == beobachter) ? null : mNewAdapter );
            EventAdapter adapter = mAdapter;
            mAdapter = mNewAdapter;
            mNewAdapter = adapter;
        }
    }

    public void loescheBeobachter()
    {
        setzeBeobachter(null);
    }
    
    public boolean pruefeAufEreignisueberlauf()
    {
        return mSensor.getAndResetOverflowFlag();
    }
}
