/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> The interface ITouchSensor extends the interface {@link ISensor}
 * and provides simplified access methods for touch sensors.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public interface ITouchSensor extends ISensor
{ 
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Returns the state of the sensor.
     * 
     * @return  true, if pressed and false, if not
     */
    public boolean isPressed();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the touch sensor is in the state <i>pressed</i>.
     */
    public void waitUntilPressed();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the touch sensor is in the state <i>released</i>.
     */
    public void waitUntilReleased();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the touch sensor is in the state <i>pressed</i> or a timeout occurs.
     * 
     * @param   timeout    the maximum time period to wait (in ms)
     * 
     * @return  true, if the sensor has been pressed or false, if not
     */
    public boolean waitUntilPressed(int timeout);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the touch sensor is in the state <i>released</i> or a timeout occurs.
     * 
     * @param   timeout    the maximum time period to wait (in ms)
     * 
     * @return  true, if the sensor has been released or false, if not
     */
    public boolean waitUntilReleased(int timeout);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the state of the touch sensor changes from <i>released</i> to <i>pressed</i>.
     */
    public void waitForPressEvent();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the state of the touch sensor changes from <i>pressed</i> to <i>released</i>.
     */
    public void waitForReleaseEvent();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the state of the touch sensor changes from <i>released</i> to <i>pressed</i> or a timeout occurs.
     * 
     * @param   timeout    the maximum time period to wait (in ms)
     * 
     * @return  true, if the sensor has been pressed or false, if not
     */
    public boolean waitForPressEvent(int timeout);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Waits until the state of the touch sensor changes from <i>pressed</i> to <i>released</i> or a timeout occurs.
     * 
     * @param   timeout    the maximum time period to wait (in ms)
     * 
     * @return  true, if the sensor has been released or false, if not
     */
    public boolean waitForReleaseEvent(int timeout);
}
