/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

import org.jabotics.robot.defs.ESensorType;
import org.jabotics.robot.defs.EButtonType;
import org.jabotics.robot.core.CSensor;
import org.jabotics.robot.core.CEventDispatcher;

/**
 * Objects of type XSensor represent a sensor with base functionality.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
class XSensor implements ISensor
{
    // Instanzvariablen
    protected CSensor mSensor; // wird in Konstruktur der abgeleiteten Klassen gesetzt

    private volatile EventAdapter mAdapter = new EventAdapter();
    private volatile EventAdapter mNewAdapter = new EventAdapter();

    private class EventAdapter implements CEventDispatcher.IHandler
    {
        private volatile ISensorListener mListener = null;

        public void manageEvent(int port, float oldValue, float newValue, float oldRate, float newRate)
        {
            mListener.manageEvent(XSensor.this,oldValue,newValue,oldRate,newRate);
        }
    }

    XSensor(org.jabotics.robot.en.ESensorPort port, ESensorType type)
    {
        mSensor = CSensor.getSensor(org.jabotics.robot.defs.ESensorPort.values()[port.ordinal()], type);
    }

    XSensor(EButtonType type)
    {
        mSensor = CSensor.getButton(type);
    }
    
    public float getMeasuredValue()
    {
        return mSensor.getOutputValue();
    }

    public float getChangeRate()
    {
        return mSensor.getChangeRate();
    }

    public float getFilterToleranceValue()
    {
        return mSensor.getEventFilterValue();
    }

    public float getFilterToleranceRate()
    {
        return mSensor.getEventFilterRate();
    }
    
    public int getIntegrationTime()
    {
        return mSensor.getIntegrationTime();
    }

    public void setFilterTolerances(float tolerance_value, float tolerance_rate)
    {
        mSensor.setEventFilter(tolerance_value, tolerance_rate);
    }

    public void setIntegrationTime(int time)
    {
        mSensor.setIntegrationTime(time);
    }

    public void waitUntilBeyondLimit(float limit)
    {
        mSensor.waitUntilBeyondLimit(limit);
    }

    public void waitUntilBelowLimit(float limit)
    {
        mSensor.waitUntilBelowLimit(limit);
    }

    public boolean waitUntilBeyondLimit(float limit, int timeout)
    {
        return mSensor.waitUntilBeyondLimit(limit, timeout);
    }

    public boolean waitUntilBelowLimit(float limit, int timeout)
    {
        return mSensor.waitUntilBelowLimit(limit, timeout);
    }

    public synchronized void setListener(ISensorListener listener)
    {
        if (listener != mAdapter.mListener)
        {
            mNewAdapter.mListener = listener;
            mSensor.setEventHandler( (null == listener) ? null : mNewAdapter );
            EventAdapter adapter = mAdapter;
            mAdapter = mNewAdapter;
            mNewAdapter = adapter;
        }
    }

    public void clearListener()
    {
        setListener(null);
    }
    
    public boolean checkForEventOverflow()
    {
        return mSensor.getAndResetOverflowFlag();
    }
}
