/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.util;


/**
 * This class provides a simple buffering mechanism for string messages.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public class XLogBuffer
{
    // Instanzvariablen
    static private StringBuilder sSB = new StringBuilder();
    static private Object sMonitor = new Object();
    static private volatile boolean sEnabled = false;

    static public void enableLogging(boolean enable)
    {
        sEnabled = enable;
    }

    static public boolean isEnabled()
    {
        return sEnabled;
    }

    static public Object getMonitor()
    {
        return sMonitor;
    }

    static public void print(String message)
    {
        if (sEnabled)
        {
            synchronized(sMonitor)
            {
                sSB.append(message);
            }
        }
    }

    static public void println(String message)
    {
        if (sEnabled)
        {
            synchronized(sMonitor)
            {
                sSB.append(message);
                sSB.append('\n');
            }
        }
    }

    static public String flush()
    {
        synchronized(sMonitor)
        {
            String s = sSB.toString();
            sSB.setLength(0);
            return s;
        }
    }
}
