/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.ISessionData;
import java.awt.Graphics2D;
import java.util.ArrayList;

/**
 * Abstrakte Klasse BModelElement - beschreiben Sie hier die Klasse
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public abstract class BModelElement implements ISessionData
{
    // Instanzvariablen
    protected ArrayList<BModelElement> mChildElements = new ArrayList<BModelElement>();
    protected XGraphicsGroup mGraphicsRep = new XGraphicsGroup();

    final public int getChildrenSize()
    {
        return mChildElements.size();
    }

    final public BModelElement getChild(int pos)
    {
        return mChildElements.get(pos);
    }

    final public void addChild(BModelElement element)
    {
        mChildElements.add(element);
    }

    final public void addChild(int pos, BModelElement element)
    {
        mChildElements.add(pos,element);
    }

    final public void setChild(int pos, BModelElement element)
    {
        mChildElements.set(pos,element);
    }

    final public void removeChild(BModelElement element)
    {
        mChildElements.remove(element);
    }

    final public void removeChild(int pos)
    {
        mChildElements.remove(pos);
    }

    final public void clearChildren()
    {
        mChildElements.clear();
    }

    final public void traverseRootLeaf(BModelVirtFuncCall vf_call)
    {
        vf_call.setSkipChildren(false);
        vf_call.execute(this);
        if (vf_call.getSkipChildren()) return;

        for (BModelElement element : mChildElements)
        {
            element.traverseRootLeaf(vf_call);
        }
    }

    final public void traverseLeafRoot(BModelVirtFuncCall vf_call)
    {
        for (BModelElement element : mChildElements)
        {
            element.traverseLeafRoot(vf_call);
        }

        vf_call.execute(this);
    }

    final public void paint(Graphics2D g)
    {
        mGraphicsRep.applyContext(g);
        mGraphicsRep.paintInLocalContext(g);
        for (BModelElement element : mChildElements)
        {
            element.paint(g);
        }
        mGraphicsRep.resetContext(g);
    }

    public void setTransform(float translationX, float translationY, float rotation)
    {
        mGraphicsRep.setTransform(translationX, translationY, rotation);
    }

    public void updateGraphics() {}
    public void updateDynamics() {}

    @Override
    public String getKey() {return this.toString();}
    @Override
    public void stopSession() {}
    @Override
    public ISessionData resetSession() {return this;}
    @Override
    public void startSession() {}

    abstract public void manageVisitor(BModelVisitor visitor);
}
