/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/

package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.util.*;
import javax.sound.sampled.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.nio.ByteBuffer;
//import java.io.File;

/**
 * Beschreiben Sie hier die Klasse XAudioSystem.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public class XAudioSystem
{
    // constants and class variables:
    static private final int MIXER_THREAD_PRIORITY = Thread.MAX_PRIORITY - 2;
    static private final int UPDATE_DELAY = 1;
    static private final float SAMPLE_RATE_IN = 5512f;
    //static private final float SAMPLE_RATE_IN = 11025f;  // one quarter of the sampling rate of audio CDs; used for lower-quality PCM and MPEG audio
    static private final int SAMPLE_SIZE_IN_BYTE = 2;    //  Signal-to-quantization-noise ratio (SQNR) for a pure sine wave of 98 dB
    static private final int SAMPLE_SIZE_IN = 8 * SAMPLE_SIZE_IN_BYTE;
    static private final AudioFormat AUDIO_FORMAT_IN = new AudioFormat(SAMPLE_RATE_IN, SAMPLE_SIZE_IN, 1, true, true);
    static private final int BUFFER_SIZE_IN = 16;
    static private final int BUFFER_SIZE_IN_BYTE = BUFFER_SIZE_IN * SAMPLE_SIZE_IN_BYTE;
    static private final double MAX_MIC_SIGNAL_LOG = Math.log10(0x800000);
    static private final int C2 = 523;
    static private final int E2 = (C2 * 5) / 4;
    static private final int G2 = (C2 * 3) / 2;
    static private final int C3 = C2 * 2;
    static private final int E3 = C2 * 5 / 2;
    static private final int G3 = C2 * 3;
    static private final int MIN_FREQUENCY = 18;
    static private final int MAX_FREQUENCY = 20000;
    static private final float MAX_AMPLITUDE = (float)Short.MAX_VALUE;
    static private final int MIN_ATTACK_MILLIS = 2;
    static private final int SAMPLE_RATE_OUT = 44100;
    static private final int SAMPLE_SIZE_OUT_BYTE = 2;
    static private final int SAMPLE_SIZE_OUT = 8 * SAMPLE_SIZE_OUT_BYTE;
    static private final double SAMPLE_TIME_OUT = 1./ (double)SAMPLE_RATE_OUT;
    static private final AudioFormat AUDIO_FORMAT_OUT = new AudioFormat((float)SAMPLE_RATE_OUT, SAMPLE_SIZE_OUT, 1, true, true);
    //static private final int BUFFER_SIZE_OUT = SAMPLE_RATE_OUT / 2; // correxponds to 0.5s
    static private final int BUFFER_SIZE_OUT = SAMPLE_RATE_OUT / 4; // correxponds to 0.25s
    static private final int BUFFER_SIZE_OUT_BYTE = BUFFER_SIZE_OUT * SAMPLE_SIZE_OUT_BYTE;
    static private final byte[] ZERO_BYTE_ARRAY = {0,0};
    /***********************_MOTOR_NOISE_START_***********************
    static private final double CLIP_TIME = 0.05;   // 1/20s
    static private final double PULSE_TIME = 0.005; // 1/200s
    static private final double PULSE_MOD_FREQUENCY = 0.5 / PULSE_TIME;
    static private final double PULSE_FREQUENCY = 30. / PULSE_TIME;
    static private final int SIZE_CLIP = (int)(CLIP_TIME * SAMPLE_RATE_OUT);
    static private final int SIZE_CLIP_BYTE = SIZE_CLIP * SAMPLE_SIZE_OUT_BYTE;
    static private final int SIZE_PULSE = (int)(PULSE_TIME * SAMPLE_RATE_OUT);
    static private final int START_PULSE = SIZE_CLIP - SIZE_PULSE;
    static private final float MAX_ROT_SPEED = 1.2f * D.TWO_PI_F * 800f / 360f;  // rad/s
    static private final float GAIN_DYNAMICS = 20f;
    static private final float GAIN_BASE = -35f;
    *************************_MOTOR_NOISE_END_***********************/

    static private volatile XPreferences.Read sLastPrefs = null;
    static private volatile boolean sInSilentMode = false;
    static private final XTimedThread sMixerThread;
    static private final MixerFeed sMixerFeed;
    static private final XTimedThread sSDLThread;
    static private final SDLFeed sSDLFeed;
    static private final CBMixerInfo CB_MIXER_INFO_OFF;
    static private final CBMixerInfo CB_MIXER_INFO_DEFAULT;
    static private final HashMap<String,CBMixerInfo> sCBMixerInfoMap = new HashMap<String,CBMixerInfo>();
    static private final ArrayList<CBMixerInfo> sCBMixerInfosIn = new ArrayList<CBMixerInfo>();
    static private final ArrayList<CBMixerInfo> sCBMixerInfosOut = new ArrayList<CBMixerInfo>();
    static private volatile int sMicSignal = 0;
    static private volatile TDL sNewTDL = null;
    static private volatile SDL sNewSDL = null;
    /***********************_MOTOR_NOISE_START_***********************
    static private volatile RClip sNewClip = null;
    *************************_MOTOR_NOISE_END_***********************/
    static private volatile TDL sActiveTDL = null;
    static private volatile SDL sActiveSDL = null;
    /***********************_MOTOR_NOISE_START_***********************
    static private volatile RClip sActiveClip = null;
    static private volatile FloatControl sGainCtrl = null;
    static private volatile long sNextClipUpdateTime = 0L;
    *************************_MOTOR_NOISE_END_***********************/
    static private volatile ToneData sToneData = null;
    static private volatile boolean sPausing = true;
    static private volatile boolean sSystemSound = false;
    static private final ByteBuffer sByteBufferIn = ByteBuffer.allocate(BUFFER_SIZE_IN_BYTE);
    static private final byte[] sByteArrayIn = sByteBufferIn.array();
    static private final ByteBuffer sByteBufferOut = ByteBuffer.allocate(BUFFER_SIZE_OUT_BYTE);
    static private final byte[] sByteArrayOut = sByteBufferOut.array();
    /***********************_MOTOR_NOISE_START_***********************
    static private final ByteBuffer sByteBufferClip = ToneGenerator.getPulseData();
    static private final XRobot sRobot = XSimulationModel.getModel().getRobot();
    *************************_MOTOR_NOISE_END_***********************/


    static
    {
        CB_MIXER_INFO_OFF = new CBMixerInfo(XPreferences.AUDIO_MIXER_OFF,XMsg.AUDIO_OFF);
        sCBMixerInfoMap.put(CB_MIXER_INFO_OFF.getKey(),CB_MIXER_INFO_OFF);
        sCBMixerInfosIn.add(CB_MIXER_INFO_OFF);
        sCBMixerInfosOut.add(CB_MIXER_INFO_OFF);

        Mixer.Info[] mixerInfos = AudioSystem.getMixerInfo();
        for (Mixer.Info mixerInfo : mixerInfos)
        {
            Mixer mixer = AudioSystem.getMixer(mixerInfo);
            MixerData md = new MixerData(mixerInfo,mixer);
            CBMixerInfo cbmi = new CBMixerInfo(md);
            sCBMixerInfoMap.put(cbmi.getKey(),cbmi);

            Line.Info[] lineInfos = mixer.getTargetLineInfo();
            for (Line.Info lineInfo : lineInfos)
            {
                try
                {
                    Line line = mixer.getLine(lineInfo);
                    if (line instanceof TargetDataLine)
                    {
                        sCBMixerInfosIn.add(cbmi);
                        break;
                    }
                }
                catch (Exception e) {}
            }

            lineInfos = mixer.getSourceLineInfo();
            for (Line.Info lineInfo : lineInfos)
            {
                try
                {
                    Line line = mixer.getLine(lineInfo);
                    if (line instanceof SourceDataLine)
                    {
                        sCBMixerInfosOut.add(cbmi);
                        break;
                    }
                }
                catch (Exception e) {}
            }
        }


        Mixer defaultMixer = AudioSystem.getMixer(null);
        if (null == defaultMixer)
        {
            CB_MIXER_INFO_DEFAULT = new CBMixerInfo(XPreferences.AUDIO_MIXER_DEFAULT,XMsg.AUDIO_MIXER_DEFAULT);

            System.out.print(XMsg.AUDIO_SYSTEM);
            System.out.print(": ");
            System.out.println(XMsg.AUDIO_MIXER_DEFAULT_INVALID);
        }
        else
        {
            Mixer.Info mi = defaultMixer.getMixerInfo();
            CBMixerInfo cbmi = sCBMixerInfoMap.get(mi.toString());
            CB_MIXER_INFO_DEFAULT = new CBMixerInfo(XPreferences.AUDIO_MIXER_DEFAULT,
                                                    XMsg.AUDIO_MIXER_DEFAULT,
                                                    (null == cbmi) ? new MixerData(mi,defaultMixer) :cbmi.getMixerData());
        }
        
        sCBMixerInfoMap.put(CB_MIXER_INFO_DEFAULT.getKey(),CB_MIXER_INFO_DEFAULT);
        sCBMixerInfosIn.add(1,CB_MIXER_INFO_DEFAULT);
        sCBMixerInfosOut.add(1,CB_MIXER_INFO_DEFAULT);

        preferencesChanged(XPreferences.getReadInstance());

        sMixerFeed = new MixerFeed();
        sMixerThread = new XTimedThread("XAudioSystem.MixerFeed",sMixerFeed);
        sMixerThread.enableTimeDilatation(false);
        sMixerThread.setDaemon(true);
        sMixerThread.setPriority(MIXER_THREAD_PRIORITY);
        sMixerThread.setTimeouts(UPDATE_DELAY,UPDATE_DELAY,0);
        sMixerThread.start();

        sSDLFeed = new SDLFeed();
        sSDLThread = new XTimedThread("XAudioSystem.SDLFeed",sSDLFeed);
        sSDLThread.enableTimeDilatation(false);
        sSDLThread.setDaemon(true);
        sSDLThread.setPriority(MIXER_THREAD_PRIORITY);
        sSDLThread.setTimeouts(UPDATE_DELAY,UPDATE_DELAY,0);
        sSDLThread.start();

        XSimulation.addListener(new PauseHandler());
        XPreferences.addListener(new PreferencesHandler());
    }


    static class CBMixerInfo
    {
        private String mKey;
        private String mName;
        private MixerData mMixerData;

        private CBMixerInfo(String key, String name)
        {
            mKey = key;
            mName = name;
            mMixerData = null;
        }

        private CBMixerInfo(String key, String name, MixerData mixerData)
        {
            mKey = key;
            mName = name;
            mMixerData = mixerData;
        }

        private CBMixerInfo(MixerData mixerData)
        {
            mKey = mixerData.toString();
            mName = mKey;
            mMixerData = mixerData;
        }

        String getKey()
        {
            return mKey;
        }
        
        String getName()
        {
            return mName;
        }

        MixerData getMixerData()
        {
            return mMixerData;
        }

        @Override
        public String toString()
        {
            return mName;
        }

        @Override
        public boolean equals(Object obj)
        {
            if ((null == obj) || !(obj instanceof CBMixerInfo)) return false;
            CBMixerInfo mixerInfo = (CBMixerInfo)obj;
            return ((mixerInfo == this) || this.mKey.equals(mixerInfo.mKey));
        }
    }

    static class MixerData
    {
        private Mixer.Info mMixerInfo;
        private Mixer mMixer;
        private TDL mTDL = null;
        private SDL mSDL = null;
        /***********************_MOTOR_NOISE_START_***********************
        private RClip mClip = null;
        *************************_MOTOR_NOISE_END_***********************/

        private MixerData(Mixer.Info mixerInfo, Mixer mixer)
        {
            mMixerInfo = mixerInfo;
            mMixer = mixer;
        }

        Mixer.Info getMixerInfo()
        {
            return mMixerInfo;
        }

        Mixer getMixer()
        {
            return mMixer;
        }

        TDL getTDL()
        {
            return mTDL;
        }

        SDL getSDL()
        {
            return mSDL;
        }

        /***********************_MOTOR_NOISE_START_***********************
        RClip getClip()
        {
            return mClip;
        }
        *************************_MOTOR_NOISE_END_***********************/

        void setTDL(TDL tdl)
        {
            mTDL = tdl;
        }

        void setSDL(SDL sdl)
        {
            mSDL = sdl;
        }

        /***********************_MOTOR_NOISE_START_***********************
        void setClip(RClip clip)
        {
            mClip = clip;
        }
        *************************_MOTOR_NOISE_END_***********************/

        @Override
        public String toString()
        {
            return mMixerInfo.toString();
        }

        @Override
        public boolean equals(Object obj)
        {
            if ((null == obj) || !(obj instanceof MixerData)) return false;
            MixerData mixerData = (MixerData) obj;
            return ((mixerData == this) || (this.toString().equals(mixerData.toString())));
        }
    }

    static private class TDL extends TResource<TargetDataLine>
    {
        TDL(TargetDataLine resource)
        {
            super(resource);
        }

        @Override
        public void acquire()
        {
            try
            {
                if (!mResource.isOpen())  mResource.open(AUDIO_FORMAT_IN);
            }
            catch (LineUnavailableException e) {}
        }

        @Override
        public void release()
        {
            if (mResource.isRunning()) mResource.stop();
            if (mResource.isOpen())  mResource.close();
        }
    }

    static private class SDL extends TResource<SourceDataLine>
    {
        SDL(SourceDataLine resource)
        {
            super(resource);
        }

        @Override
        public void acquire()
        {
            try
            {
                if (!mResource.isOpen())
                {
                    mResource.open(AUDIO_FORMAT_OUT);
                }
                if (!mResource.isRunning())
                {
                    mResource.start();
                    // The following commands are necessary to make the sdl completely operational:
                    mResource.write(ZERO_BYTE_ARRAY,0,2);
                    mResource.drain();
                }
            }
            catch (LineUnavailableException e) {}
        }

        @Override
        public void release()
        {
            if (mResource.isRunning())
            {
                mResource.flush();
                mResource.stop();
            }
            if (mResource.isOpen())    mResource.close();
        }
    }

    /***********************_MOTOR_NOISE_START_***********************
    static private class RClip extends Resource<Clip>
    {
        RClip(Clip resource)
        {
            super(resource);
        }

        @Override
        public void acquire()
        {
            try
            {
                if (!mResource.isOpen())
                {
                    mResource.open(AUDIO_FORMAT_OUT,sByteBufferClip.array(),0,SIZE_CLIP_BYTE);
                }
                if (!mResource.isRunning())
                {
                    Control ctrl = mResource.getControl(FloatControl.Type.MASTER_GAIN);
                    FloatControl gainCtrl = (ctrl instanceof FloatControl) ? (FloatControl)ctrl : null;
                    if (null != gainCtrl) gainCtrl.setValue(gainCtrl.getMinimum());
                    mResource.setLoopPoints(0,START_PULSE);
                    mResource.loop(Clip.LOOP_CONTINUOUSLY);
                }
            }
            catch (LineUnavailableException e) {}
        }

        @Override
        public void release()
        {
            if (mResource.isRunning()) mResource.stop();
            if (mResource.isOpen())    mResource.close();
        }
    }
    *************************_MOTOR_NOISE_END_***********************/

    static private class MixerFeed implements XTimedThread.Runnable
    {
        @Override
        public void init()
        {
        }

        @Override
        public void runTimed()
        {
            while (sNewTDL != sActiveTDL)
            {
                TDL newTDL = sNewTDL;
                
                if (newTDL != sActiveTDL)
                {
                    if (null != newTDL)
                    {
                        newTDL.addRef();
                        if (newTDL.get().isOpen())
                        {
                            newTDL.get().start();
                        }
                        else
                        {
                            // sNewTDL must have been replaced in the meantime
                            newTDL.removeRef();
                            newTDL = null;
                        }
                    }
                    if (null != sActiveTDL) sActiveTDL.removeRef();
                    sActiveTDL = newTDL;
                }
            }

            if (null != sActiveTDL)
            {
                while(sActiveTDL.get().available() >= BUFFER_SIZE_IN_BYTE)
                {
                    sActiveTDL.get().read(sByteArrayIn,0,BUFFER_SIZE_IN_BYTE);
                    for (int i = 0; i < BUFFER_SIZE_IN; ++i)
                    {
                        short signal = sByteBufferIn.getShort(i+i);
                        int signal_abs = Math.abs(signal) * 0x100;
                        sMicSignal = (sMicSignal * 99 + signal_abs) / 100;
                    }
                }
            }

            /***********************_MOTOR_NOISE_START_***********************
            while (sNewClip != sActiveClip)
            {
                RClip newClip = sNewClip;
                
                if (newClip != sActiveClip)
                {
                    FloatControl newGainCtrl = null;
                    if (null != newClip)
                    {
                        newClip.addRef();
                        if (newClip.get().isOpen())
                        {
                            Control ctrl = newClip.get().getControl(FloatControl.Type.MASTER_GAIN);
                            if (ctrl instanceof FloatControl) newGainCtrl = (FloatControl)ctrl;
                        }
                        else
                        {
                            // sNewClip must have been replaced in the meantime
                            newClip.removeRef();
                            newClip = null;
                        }
                    }
                    if (null != sActiveClip) sActiveClip.removeRef();
                    sActiveClip = newClip;
                    sGainCtrl = newGainCtrl;
                }
            }

            if (null != sActiveClip)
            {
                Bei Aktivierung der Motorengeräusche muss MixerFeed von ISessionData abgeleitet
                und bei XSession angemeldet werden, damit bei einem Neustart der Controller-Thread
                vorübergehend "ausgesetzt" und sNextClipUpdateTime zurückgesetzt werden kann.
                long currentTime = XTimer.currentTimeMillis();
                if (currentTime >= sNextClipUpdateTime)
                {
                    sNextClipUpdateTime = currentTime + 50;

                    Clip clip = sActiveClip.get();
                    XRobot.XDynamicData dd = sRobot.getDynamicData();
                    float speedLeft = Math.abs(dd.getRotSpeedLeft());
                    float speedRight = Math.abs(dd.getRotSpeedRight());
                    float weightLeft = speedLeft * speedLeft;
                    float weightRight = speedRight * speedRight;
                    float weight = weightLeft + weightRight;
                    float relSpeed = 0f;
                    int start = -1;
                    if (weight > 0)
                    {
                        relSpeed = (speedLeft * weightLeft + speedRight * weightRight) / (MAX_ROT_SPEED * weight);
                        start = SIZE_CLIP - (int)((float)SIZE_PULSE / relSpeed) - 1;
                    }
                    if ((start < 0) || sPausing)
                    {
                        if (null != sGainCtrl) sGainCtrl.setValue(sGainCtrl.getMinimum());
                        clip.setLoopPoints(0,START_PULSE);
                        //if (clip.isRunning())
                        //{
                        //    clip.stop();
                        //}
                    }
                    else
                    {
                        clip.setLoopPoints(start,-1);
                        if (null != sGainCtrl)
                        {
                            sGainCtrl.setValue(GAIN_BASE + GAIN_DYNAMICS * relSpeed);
                        }
                        //if (!clip.isRunning())
                        //{
                        //    clip.loop(Clip.LOOP_CONTINUOUSLY);
                        //}
                    }
                }
            }
            *************************_MOTOR_NOISE_END_***********************/
        }

        @Override
        public void terminate()
        {
            // nothing to do since runTimed() should finish quickly
        }

        @Override
        public void shutdown()
        {
            if (null != sNewTDL) while (sNewTDL.getRefCount() > 0) sNewTDL.removeRef();
            if (null != sActiveTDL) while (sActiveTDL.getRefCount() > 0) sActiveTDL.removeRef();
            /***********************_MOTOR_NOISE_START_***********************
            if (null != sNewClip) while (sNewClip.getRefCount() > 0) sNewClip.removeRef();
            if (null != sActiveClip) while (sActiveClip.getRefCount() > 0) sActiveClip.removeRef();
            *************************_MOTOR_NOISE_END_***********************/
        }
    }

    private static class SDLFeed implements XTimedThread.Runnable
    {
        //private volatile Thread mThread;

        @Override
        public void init()
        {
            //mThread = Thread.currentThread();
        }

        @Override
        public void runTimed()
        {
            if (null != sToneData)
            {
                playTone(sToneData);
                sToneData = null;
            }
        }

        @Override
        public void terminate()
        {
            //mThread.interrupt(); already done by XTimedThread
        }

        @Override
        public void shutdown()
        {
            if (null != sNewSDL) while (sNewSDL.getRefCount() > 0) sNewSDL.removeRef();
            if (null != sActiveSDL) while (sActiveSDL.getRefCount() > 0) sActiveSDL.removeRef();
        }

        private void playTone(ToneData td)
        {
            SourceDataLine sdl = sActiveSDL.get();

            checkPausing();
            if (checkTerminate()) return;
    
            long n_samples = ToneGenerator.initTone(td);
            if (0 == n_samples) return;
            long targetFrame = sdl.getLongFramePosition() + n_samples;
            long remaining_samples = n_samples;
            while(remaining_samples > 0)
            {
                int next_samples = (remaining_samples < BUFFER_SIZE_OUT) ? (int)remaining_samples : BUFFER_SIZE_OUT;
                int next_bytes = next_samples * SAMPLE_SIZE_OUT_BYTE;
                remaining_samples = ToneGenerator.getToneData(sByteBufferOut,0,next_samples);
                while (sdl.available() < next_bytes)
                {
                    checkPausing();
                    if (checkTerminate()) return;
                    XTimer.msDelay(1);
                    if (checkTerminate()) return;
                }
                checkPausing();
                if (checkTerminate()) return;
                sdl.write(sByteArrayOut,0,next_bytes);
                if (checkTerminate()) return;
            }
            while (sdl.getLongFramePosition() < targetFrame)
            {
                checkPausing();
                if (checkTerminate()) return;
                XTimer.msDelay(1);
                if (checkTerminate()) return;
            }
            sdl.drain();
        }

        private boolean checkTerminate()
        {
            //if (mThread.isInterrupted())
            if (Thread.currentThread().isInterrupted())
            {
                if (sActiveSDL.get().isRunning()) sActiveSDL.get().stop();
                sActiveSDL.get().flush();
                return true;
            }

            return false;
        }

        private void checkPausing()
        {
            if (sPausing && !sSystemSound)
            {
                sActiveSDL.get().stop();
                while (sPausing)
                {
                    XTimer.msDelay(1);
                    //if (mThread.isInterrupted()) return;
                    if (Thread.currentThread().isInterrupted()) return;
                }
                sActiveSDL.get().start();
            }
        }
    }

    static private class PauseHandler implements XSimulation.IListener
    {
        @Override
        public void stateChanged(XSimulation.EState oldState, XSimulation.EState newState)
        {
            sPausing = ((XSimulation.EState.PAUSING == newState) || (XSimulation.EState.TERMINATING == newState));
        }
    }

    static private class PreferencesHandler implements XPreferences.IListener
    {
        public void preferencesChanged(XPreferences.Read newPrefs)
        {
            XAudioSystem.preferencesChanged(newPrefs);
        }
    }
    static CBMixerInfo[] getCBMixerInfosIn()
    {
        return sCBMixerInfosIn.toArray(new CBMixerInfo[sCBMixerInfosIn.size()]);
    }

    static CBMixerInfo[] getCBMixerInfosOut()
    {
        return sCBMixerInfosOut.toArray(new CBMixerInfo[sCBMixerInfosOut.size()]);
    }

    static CBMixerInfo getCBMixerInfo(String key)
    {
        if (XPreferences.AUDIO_MIXER_OFF.equals(key)) return CB_MIXER_INFO_OFF;
        if (XPreferences.AUDIO_MIXER_DEFAULT.equals(key)) return CB_MIXER_INFO_DEFAULT;
        CBMixerInfo cbmi = sCBMixerInfoMap.get(key);
        return (null == cbmi) ? CB_MIXER_INFO_DEFAULT : cbmi;
    }

    static void enableSilentMode(boolean enabled)
    {
        sInSilentMode = enabled;
    }

    // no synchronization necessary (done in XPreferences)
    static private void preferencesChanged(XPreferences.Read newPrefs)
    {
        String audioMixerIn = newPrefs.getAudioMixerIn();
        String audioMixerOut = newPrefs.getAudioMixerOut();

        if ( (null == sLastPrefs) || !audioMixerIn.equals(sLastPrefs.getAudioMixerIn()) )
        {
            TDL previousTDL = sNewTDL;
            TDL newTDL = null;

            if (!audioMixerIn.equals(XPreferences.AUDIO_MIXER_OFF))
            {
                String invalidMsg = audioMixerIn.equals(XPreferences.AUDIO_MIXER_DEFAULT) ?
                                    XMsg.AUDIO_MIXER_DEFAULT_INVALID :
                                    XMsg.AUDIO_MIXER_SELECT_INVALID;

                CBMixerInfo cbmi = sCBMixerInfoMap.get(audioMixerIn);
                MixerData md = (null == cbmi) ? null : cbmi.getMixerData();
                if (null == md)
                {
                    System.out.print(XMsg.AUDIO_SYSTEM);
                    System.out.print(" / ");
                    System.out.print(XMsg.AUDIO_IN);
                    System.out.print(": ");
                    System.out.println(invalidMsg);
                }
                else
                {
                    if (null != md.getTDL())
                    {
                        // try to reopen the data line:
                        md.getTDL().addRef();
                        if (!md.getTDL().get().isOpen())
                        {
                            md.getTDL().removeRef();
                            md.setTDL(null);
                        }
                    }

                    if (null == md.getTDL())
                    {
                        // try to get a valid data line:
                        md.setTDL(getTDL(md.getMixer()));
                        if (null == md.getTDL())
                        {
                            System.out.print(XMsg.AUDIO_SYSTEM);
                            System.out.print(" / ");
                            System.out.print(XMsg.AUDIO_IN);
                            System.out.print(": ");
                            System.out.println(invalidMsg);
                        }
                    }

                    newTDL = md.getTDL();
                }
            }

            sNewTDL = newTDL;
            if (null != previousTDL) previousTDL.removeRef();
        }

        if ( (null == sLastPrefs) || !audioMixerOut.equals(sLastPrefs.getAudioMixerOut()) )
        {
            SDL previousSDL = sNewSDL;
            SDL newSDL = null;
            /***********************_MOTOR_NOISE_START_***********************
            RClip previousClip = sNewClip;
            RClip newClip = null;
            *************************_MOTOR_NOISE_END_***********************/

            if (!audioMixerOut.equals(XPreferences.AUDIO_MIXER_OFF))
            {
                String invalidMsg = audioMixerOut.equals(XPreferences.AUDIO_MIXER_DEFAULT) ?
                                    XMsg.AUDIO_MIXER_DEFAULT_INVALID :
                                    XMsg.AUDIO_MIXER_SELECT_INVALID;

                CBMixerInfo cbmi = sCBMixerInfoMap.get(audioMixerOut);
                MixerData md = (null == cbmi) ? null : cbmi.getMixerData();
                if (null == md)
                {
                    System.out.print(XMsg.AUDIO_SYSTEM);
                    System.out.print(" / ");
                    System.out.print(XMsg.AUDIO_IN);
                    System.out.print(": ");
                    System.out.println(invalidMsg);
                }
                else
                {
                    if (null != md.getSDL())
                    {
                        // try to reopen the data line:
                        md.getSDL().addRef();
                        if (!md.getSDL().get().isOpen())
                        {
                            md.getSDL().removeRef();
                            md.setSDL(null);
                        }
                    }

                    if (null == md.getSDL())
                    {
                        // try to get a valid data line:
                        md.setSDL(getSDL(md.getMixer()));
                        if (null == md.getSDL())
                        {
                            System.out.print(XMsg.AUDIO_SYSTEM);
                            System.out.print(" / ");
                            System.out.print(XMsg.AUDIO_IN);
                            System.out.print(": ");
                            System.out.println(invalidMsg);
                        }
                    }

                    newSDL = md.getSDL();

                    /***********************_MOTOR_NOISE_START_***********************
                    if (null != md.getClip())
                    {
                        // try to reopen the data line:
                        md.getClip().addRef();
                        if (!md.getClip().get().isOpen())
                        {
                            md.getClip().removeRef();
                            md.setClip(null);
                        }
                    }

                    if (null == md.getClip())
                    {
                        // try to get a valid data line:
                        md.setClip(getClip(md.getMixer()));
                        if (null == md.getClip())
                        {
                            System.out.print(XMsg.AUDIO_SYSTEM);
                            System.out.print(" / ");
                            System.out.print(XMsg.AUDIO_IN);
                            System.out.print(": ");
                            System.out.println(invalidMsg);
                        }
                    }

                    newClip = md.getClip();
                    *************************_MOTOR_NOISE_END_***********************/
                }
            }

            sNewSDL = newSDL;
            if (null != previousSDL) previousSDL.removeRef();
            /***********************_MOTOR_NOISE_START_***********************
            sNewClip = newClip;
            if (null != previousClip) previousClip.removeRef();
            *************************_MOTOR_NOISE_END_***********************/
        }

        sLastPrefs = newPrefs;
    }

    static private TDL getTDL(Mixer mixer)
    {
        if (null == mixer) return null;

        Line.Info[] lineInfos = mixer.getTargetLineInfo();
        for (Line.Info lineInfo : lineInfos)
        {
            try
            {
                Line line = mixer.getLine(lineInfo);
                if (line instanceof TargetDataLine)
                {
                    TargetDataLine tdl = (TargetDataLine)line;
                    tdl.open(AUDIO_FORMAT_IN);
                    TDL tdlRes = new TDL(tdl);
                    tdlRes.addRef();
                    return tdlRes;
                }
            }
            catch (Exception e)
            {
            }
        }
        return null;
    }

    static private SDL getSDL(Mixer mixer)
    {
        if (null == mixer) return null;

        Line.Info[] lineInfos = mixer.getSourceLineInfo();
        for (Line.Info lineInfo : lineInfos)
        {
            try
            {
                Line line = mixer.getLine(lineInfo);
                if (line instanceof SourceDataLine)
                {
                    SourceDataLine sdl = (SourceDataLine)line;
                    sdl.open(AUDIO_FORMAT_OUT);
                    SDL sdlRes = new SDL(sdl);
                    sdlRes.addRef();
                    return sdlRes;
                }
            }
            catch (Exception e)
            {
            }
        }
        return null;
    }

    /***********************_MOTOR_NOISE_START_***********************
    static private RClip getClip(Mixer mixer)
    {
        if (null == mixer) return null;

        Line.Info[] lineInfos = mixer.getSourceLineInfo();
        for (Line.Info lineInfo : lineInfos)
        {
            try
            {
                Line line = mixer.getLine(lineInfo);
                if (line instanceof Clip)
                {
                    Clip clip = (Clip)line;
                    if (!clip.isOpen())
                    {
                        RClip clipRes = new RClip(clip);
                        clipRes.addRef();
                        return clipRes;
                    }
                }
            }
            catch (Exception e)
            {
            }
        }
        return null;
    }
    *************************_MOTOR_NOISE_END_***********************/

    static private class ToneData
    {
        private int mFrequency;
        private int mModulationFrequency;
        private float mModulationDepth;
        private int mLength;
        private float mRelVolume;
        private int mAttackMillis;
        private int mDecayMillis;
        private float mSustainLevel;
        
        ToneData(int frequency, int modulation_frequency, float modulation_depth, int length, float rel_volume, int attack_millis, int decay_millis, float sustain_level)
        {
            mFrequency = frequency;
            mModulationFrequency = modulation_frequency;
            mModulationDepth = modulation_depth;
            mLength = length;
            mRelVolume = rel_volume;
            mAttackMillis = attack_millis;
            mDecayMillis = decay_millis;
            mSustainLevel = sustain_level;
        }
    }

    static private class ToneGenerator
    {
        static private final int RES_BY_4 = 255;
        static private final int RES = RES_BY_4 * 4;
        static private final double RES_DBL = (double) RES;
        static private final double[] TRIGONOMETRIC_REF = new double[RES_BY_4 + 1];
        static private final double TAYLOR1 = D.TWO_PI;
        static private final double TAYLOR2 = TAYLOR1 * D.TWO_PI / 2.;
        static private final double TAYLOR3 = TAYLOR2 * D.TWO_PI / 3.;
        //static private final double TAYLOR4 = TAYLOR3 * D.TWO_PI / 4.;
        static
        {
            double f = D.TWO_PI / RES_DBL;
            for (int i = 0;  i <= RES_BY_4;  ++i)
            {
                TRIGONOMETRIC_REF[i] = Math.sin(f*i);
            }
        }
    
        static private float sAttack = 1f;
        static private float sAttackFactor = 0f;
        static private float sSustain = 0f;
        static private float sDecay = 1f;
        static private float sDecayFactor = 1f;
        static private float sMute = 1f;
        static private long sNSamples = 0;
        static private long sRemainingSamples = 0;
        static private long sPosSamples = 0;
        static private float sAmplitude = MAX_AMPLITUDE;
        static private double sPeriodStep = 0.;
        static private double sPeriods = 0.;
        static private double sModPeriodStep = 0.;
        static private double sModPeriods = 0.;
        static private float sModAmplitude = 0f;
        static private float sModMean = 1f;

        static long initTone(ToneData td)
        {
            if (td.mLength <= 0) return 0;
    
            sByteBufferOut.clear();
    
            if (td.mRelVolume < 0f)
            {
                td.mRelVolume = 0;  // allow this to turn off a tone
            }
            else if (td.mRelVolume > 1f)
            {
                td.mRelVolume = 1f;
            }
    
            if (td.mFrequency < MIN_FREQUENCY)
            {
                td.mFrequency = MIN_FREQUENCY;
            }
            else if (td.mFrequency > MAX_FREQUENCY)
            {
                td.mFrequency = MAX_FREQUENCY;
            }
    
            if (td.mAttackMillis < MIN_ATTACK_MILLIS)  
            {
                td.mAttackMillis = MIN_ATTACK_MILLIS;
            }
    
    
            if (td.mDecayMillis < 0)
            {
                td.mDecayMillis = 0;
            }
    
            if (0 == td.mDecayMillis) td.mSustainLevel = 1f;
            else if (td.mSustainLevel < 0) td.mSustainLevel = 0f;
            else if (td.mSustainLevel > 1f) td.mSustainLevel = 1f;
    
            sAttack = 1f;
            sAttackFactor = 1f - (float)SAMPLE_TIME_OUT / (0.001f * td.mAttackMillis);
            sSustain = td.mSustainLevel;
            sDecay = 1f - td.mSustainLevel;
            sDecayFactor = (0 == td.mDecayMillis) ? 1f : 1f - (float)SAMPLE_TIME_OUT / (0.001f * td.mDecayMillis);
            sMute = 1f;
            sNSamples = ((long)SAMPLE_RATE_OUT * (long)td.mLength) / 1000;
            sRemainingSamples = sNSamples;
            sPosSamples = 0;
            sAmplitude = MAX_AMPLITUDE * td.mRelVolume; // relative volume; value between 0 and 1
            sPeriodStep = (double)td.mFrequency * SAMPLE_TIME_OUT;
            sPeriods = 0.;
            sModPeriodStep = (double)td.mModulationFrequency * SAMPLE_TIME_OUT;
            sModPeriods = 0.;
            sModAmplitude = 0.5f * td.mModulationDepth;
            sModMean = 1f - sModAmplitude;

            return sNSamples;
        }
    
        static long getToneData(ByteBuffer buffer, int pos_buffer, int samples)
        {
            for (int i = 0;  (sPosSamples < sNSamples) && (i < samples);  ++i, ++sPosSamples, pos_buffer += SAMPLE_SIZE_OUT_BYTE)
            {
                sAttack *= sAttackFactor;
                sDecay *= sDecayFactor;
                sRemainingSamples = sNSamples - sPosSamples;
                if (sRemainingSamples < 1000)
                {
                    float r1 = (float)sRemainingSamples;
                    float r2 = 1000f - r1;
                    float w1 = r1 * r1;
                    float w2 = r2 * r2;
                    sMute = w1 / (w1 + w2);
                }
    
                sPeriods += sPeriodStep;
                float amp = (1f - sAttack) * (sSustain + sDecay) * sMute * sAmplitude * sin(sPeriods);
                if (sModPeriodStep > 0)
                {
                    sModPeriods += sModPeriodStep;
                    amp *= sModMean + sModAmplitude * sin(sModPeriods);
                }
                int signal = Math.round(amp);
                if (SAMPLE_SIZE_OUT_BYTE == 1)
                {
                    buffer.put(pos_buffer,(byte)signal);
                }
                else
                {
                    buffer.putShort(pos_buffer,(short)signal);
                }
            }

            sRemainingSamples = sNSamples - sPosSamples;
            return sRemainingSamples;
        }

        /***********************_MOTOR_NOISE_START_***********************
        static ByteBuffer getPulseData()
        {
            ByteBuffer buffer = ByteBuffer.allocate(SIZE_CLIP_BYTE);

            double periodStep = PULSE_FREQUENCY * SAMPLE_TIME_OUT;
            double periods = 0.;
            double modPeriodStep = PULSE_MOD_FREQUENCY * SAMPLE_TIME_OUT;
            double modPeriods = 0;

            int pos_buffer = 0;
            for (int i = 0;  i < START_PULSE;  ++i, pos_buffer += SAMPLE_SIZE_OUT_BYTE)
            {
                if (SAMPLE_SIZE_OUT_BYTE == 1)
                {
                    buffer.put(pos_buffer,(byte)0);
                }
                else
                {
                    buffer.putShort(pos_buffer,(short)0);
                }
            }
            
            for (int i = 0;  i < SIZE_PULSE;  ++i, ++i, pos_buffer += SAMPLE_SIZE_OUT_BYTE)
            {
                periods += periodStep;
                modPeriods += modPeriodStep;
                float amp = MAX_AMPLITUDE * sin(periods) * sin(modPeriods);
                int signal = Math.round(amp);
                if (SAMPLE_SIZE_OUT_BYTE == 1)
                {
                    buffer.put(pos_buffer,(byte)signal);
                }
                else
                {
                    buffer.putShort(pos_buffer,(short)signal);
                }
            }

            return buffer;
        }
        *************************_MOTOR_NOISE_END_***********************/

        static private float sin(double periods)
        {
            /**********************************************************
             * Execution time about 30% of Math.sin(double x)
             *  on reference machine.
             * Accurate except for error by 1 for last digit in
             *  rare cases due to rounding. This corresponds to a
             *  maximum value for the absolute error of about 1E-7.
             * 
             * Approach:
             * Taylor expansion of sin starting from two different
             * neighboring reference points embracing the value of
             * periods. Intermediate values will be further smoothened
             * by calculating a weighted average of the two sin-values.
             *********************************************************/
    
            double SIN1;
            double COS1;
            double SIN2;
            double COS2;
            periods = periods - Math.floor(periods);
            double floor = Math.floor(periods * RES_DBL);
            int pos = (int)floor;
            if (pos < RES_BY_4)
            {
                int pos2 = pos + 1;
                SIN1 = TRIGONOMETRIC_REF[pos];
                SIN2 = TRIGONOMETRIC_REF[pos2];
                COS1 = TRIGONOMETRIC_REF[RES_BY_4 - pos];
                COS2 = TRIGONOMETRIC_REF[RES_BY_4 - pos2];
            }
            else
            {
                pos -= RES_BY_4;
                if (pos < RES_BY_4)
                {
                    int pos2 = pos + 1;
                    SIN1 =  TRIGONOMETRIC_REF[RES_BY_4 - pos];
                    SIN2 =  TRIGONOMETRIC_REF[RES_BY_4 - pos2];
                    COS1 = -TRIGONOMETRIC_REF[pos];
                    COS2 = -TRIGONOMETRIC_REF[pos2];
                }
                else
                {
                    pos -= RES_BY_4;
                    if (pos < RES_BY_4)
                    {
                        int pos2 = pos + 1;
                        SIN1 = -TRIGONOMETRIC_REF[pos];
                        SIN2 = -TRIGONOMETRIC_REF[pos2];
                        COS1 = -TRIGONOMETRIC_REF[RES_BY_4 - pos];
                        COS2 = -TRIGONOMETRIC_REF[RES_BY_4 - pos2];
                    }
                    else
                    {
                        pos -= RES_BY_4;
                        int pos2 = pos + 1;
                        SIN1 = -TRIGONOMETRIC_REF[RES_BY_4 - pos];
                        SIN2 = -TRIGONOMETRIC_REF[RES_BY_4 - pos2];
                        COS1 =  TRIGONOMETRIC_REF[pos];
                        COS2 =  TRIGONOMETRIC_REF[pos2];
                    }
                }
            }
            double dx1 = periods - floor / RES_DBL;
            double dx2 = periods - (floor + 1.) / RES_DBL;
            double dx1p2 = dx1 * dx1;
            double dx2p2 = dx2 * dx2;
            double dx1p3 = dx1p2 * dx1;
            double dx2p3 = dx2p2 * dx2;
            double dx1p6 = dx1p3 * dx1p3;
            double dx2p6 = dx2p3 * dx2p3;
            double sin1 = SIN1 + TAYLOR1 * dx1 * COS1 - TAYLOR2 * dx1p2 * SIN1 - TAYLOR3 * dx1p3 * COS1;
            double sin2 = SIN2 + TAYLOR1 * dx2 * COS2 - TAYLOR2 * dx2p2 * SIN2 - TAYLOR3 * dx2p3 * COS2;
            return (float) ((sin1 * dx2p6 + sin2 * dx1p6) / (dx1p6 + dx2p6));
        }
    }


    static int getMicSignal()
    {
        // internal (member mMicSignal): linear scale
        // external (return value of getMicSignal): log scale (range 0 to 100)

        // return value of -1 indicates that mic signal is not available:
        if (null == sActiveTDL)  return -1;
        
        int micSignal = sMicSignal;
        if (micSignal <= 0) return 0;

        int logSignal = (int) (100. + 40. * (Math.log10((double)micSignal) - MAX_MIC_SIGNAL_LOG)); // results in dynamic range of about 50dB (20dB corresponds to linear factor of 100)
        return (logSignal < 0) ? 0 : ( (logSignal > 100) ? 100 : logSignal);
    }


    static void tone(int frequency, int length, float rel_volume)
    {
        modulatedTone(frequency, 0, 0f, length, rel_volume, 2 * MIN_ATTACK_MILLIS, 0, 1f);
    }


    static void tone(int frequency, int length, float rel_volume, int attack_millis, int decay_millis, float sustain_level)
    {
        modulatedTone(frequency, 0, 0f, length, rel_volume, attack_millis, decay_millis, sustain_level);
    }


    static void modulatedTone(int frequency, int modulation_frequency, float modulation_depth, int length, float rel_volume)
    {
        //modulatedTone(frequency, modulation_frequency, length, rel_volume, MIN_ATTACK_MILLIS, 0);
        modulatedTone(frequency, modulation_frequency, modulation_depth, length, rel_volume, 2 * MIN_ATTACK_MILLIS, 0, 1f);
    }


    synchronized static void modulatedTone(int frequency, int modulation_frequency, float modulation_depth, int length, float rel_volume, int attack_millis, int decay_millis, float sustain_level)
    {
        if ((null != sToneData) || (length <= 0)) return;

        while (sNewSDL != sActiveSDL)
        {
            SDL newSDL = sNewSDL;
            
            if (newSDL != sActiveSDL)
            {
                if (null != newSDL)
                {
                    newSDL.addRef();
                    if (!newSDL.get().isOpen())
                    {
                        // sNewSDL must have been replaced in the meantime
                        newSDL.removeRef();
                        newSDL = null;
                    }
                }
                if (null != sActiveSDL) sActiveSDL.removeRef();
                sActiveSDL = newSDL;
            }
        }

        if ((null != sActiveSDL) && !sInSilentMode)
        {
            sToneData = new ToneData(frequency, modulation_frequency, modulation_depth, length, rel_volume, attack_millis, decay_millis, sustain_level);
        }

        if (sSystemSound)
        {
            XTimer.msDelay(length);
            while (null != sToneData) XTimer.msDelay(1);
        }
        else
        {
            XTimer.msDelayRobot(length);
            while (null != sToneData) XTimer.msDelayRobot(1);
        }
    }

    
    synchronized static void playRobotSignal()
    {
        sSystemSound = true;
        float rel_volume = 0.75f;
        modulatedTone(C2, C3, 0.2f, 400, rel_volume, 6, 200, 0f);
        modulatedTone(G2, G3, 0.2f, 400, rel_volume, 6, 200, 0f);
        modulatedTone(E2, E3, 0.2f, 400, rel_volume, 6, 200, 0f);
        sSystemSound = false;
    }


    /*static public void testDefaultMixer()
    {
        StringBuilder sb = new StringBuilder();
        Mixer mixer = AudioSystem.getMixer(null);
        logInfo(mixer,sb);
        System.out.print(sb.toString());
    }

    static void logInfo(Mixer mixer, StringBuilder sb)
    {
        Line.Info[] lineInfos;

        sb.append("\n**************************************************\n");
        sb.append(mixer.toString());
        sb.append("\n**************************************************\n");
        Mixer.Info mixerInfo = mixer.getMixerInfo();
        sb.append(" Name:      ");
        sb.append(mixerInfo.getName());
        sb.append("\n Vendor:    ");
        sb.append(mixerInfo.getName());
        sb.append("\n Version:   ");
        sb.append(mixerInfo.getName());
        sb.append("\n Descript.: ");
        sb.append(mixerInfo.getName());
        sb.append("\n**************************************************\n");
        lineInfos = mixer.getSourceLineInfo();
        for(Line.Info lineInfo : lineInfos)
        {
            sb.append(' ');
            sb.append(lineInfo.toString());
            sb.append('\n');
            try
            {
                Line line = mixer.getLine(lineInfo);
                if (line instanceof Port) sb.append(" ... implementiert Schnittstelle Port.\n");
                if (line instanceof Clip) sb.append(" ... implementiert Schnittstelle Clip.\n");
                if (line instanceof SourceDataLine) sb.append(" ... implementiert Schnittstelle SourceDataLine.\n");
                if (line instanceof TargetDataLine) sb.append(" ... implementiert Schnittstelle TargetDataLine.\n");
            }
            catch (LineUnavailableException e)
            {
                sb.append(" ... nicht verfügbar\n");
            }
            
        }
        sb.append(" ------\n");
        lineInfos = mixer.getTargetLineInfo();
        for(Line.Info lineInfo : lineInfos)
        {
            sb.append(' ');
            sb.append(lineInfo.toString());
            sb.append('\n');
            try
            {
                Line line = mixer.getLine(lineInfo);
                sb.append('\n');
                if (line instanceof Port) sb.append(" ... implementiert Schnittstelle Port.\n");
                if (line instanceof Clip) sb.append(" ... implementiert Schnittstelle Clip.\n");
                if (line instanceof SourceDataLine) sb.append(" ... implementiert Schnittstelle SourceDataLine.\n");
                if (line instanceof TargetDataLine) sb.append(" ... implementiert Schnittstelle TargetDataLine.\n");
            }
            catch (LineUnavailableException e)
            {
                sb.append(" ... nicht verfügbar\n");
            }
            
        }
        
        sb.append("**************************************************\n");
    }

    static public void testSinAccuracy()
    {
        float max_error = 0f;
        for (int i = 0;  i < 1000000;  ++i)
        {
            double periods = Math.random();
            float s1 = (float)Math.sin(D.TWO_PI * periods);
            float s2 = sin(periods);
            float error = (s1 > s2) ? s1 - s2 : s2 - s1;
            if (max_error < error) max_error = error;
        }
    }

    static public void testSinExecutionTime()
    {
        final int N = 10000000;
        float sum1 = 0;
        long start1 = XTimer.currentTimeMillis();
        for (int i = 0;  i < N;  ++i)
        {
            sum1 += (float)Math.sin(Math.random()*D.TWO_PI);
        }
        long stop1 = XTimer.currentTimeMillis();
        long time1 = stop1 - start1;
        float sum2 = 0;
        long start2 = XTimer.currentTimeMillis();
        for (int i = 0;  i < N;  ++i)
        {
            sum2 += sin(Math.random()*3.0);
        }
        long stop2 = XTimer.currentTimeMillis();
        long time2 = stop2 - start2;
        float sum3 = 0;
        long start3 = XTimer.currentTimeMillis();
        for (int i = 0;  i < N;  ++i)
        {
            sum3 += Math.random()*D.TWO_PI;
        }
        long stop3 = XTimer.currentTimeMillis();
        long time3 = stop3 - start3;
        time1 -= time3;
        time2 -= time3;
        double rel = (double)time2 / (double)time1;
    }*/

    static public void testSDL()
    {
        sPausing = false;
        XTimer.resumeRobot();
        
        XTimer.msDelay(2000);
        
        playRobotSignal();

        XTimer.msDelay(2000);
        
        tone(440, 2000, 0.6f, 20, 0, 1f);

        XTimer.msDelay(2000);
        
        tone(8440, 2000, 1f, 10, 300, 0.1f);
        
        XTimer.msDelay(2000);
        
        tone(2560, 2000, 0.8f, 10, 1000, 0.2f);
        
        XTimer.msDelay(2000);
        
        tone(440, 2000, 0.8f, 50, 5000, 0f);
        
        XTimer.msDelay(2000);
        
        tone(130, 2000, 0.8f, 50, 500, 0f);
    }
}
