/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.swing;

import java.io.*;
import java.awt.*;
import javax.swing.*;

/**
 * Use XOutErrStreamPanel to log an out stream and an error stream on the screen.
 * 
 * @author  Frank Gollnik
 * @version 1.1.1
 */
public class XOutErrStreamPanel extends JPanel implements XLogStreamTextArea.ILogListener
{
    // instance variables
    private boolean mHasOutput = false;
    private boolean mHasError = false;
    private XLogStreamTextArea mOutTextArea;
    private XLogStreamTextArea mErrTextArea;
    private JSplitPane mSplitPane;

    /**
     * Konstruktor für Objekte der Klasse XOutErrStreamPanel
     */
    public XOutErrStreamPanel()
    {
        super(new GridBagLayout());
        /*init();
    }

    public XOutErrStreamPanel(Dimension preferredSize)
    {
        super(new GridBagLayout());
        init();
        top.setPreferredSize(preferredSize);
        buttom.setPreferredSize(preferredSize);
    }

    private init()
    {*/
        mOutTextArea = new XLogStreamTextArea();
        mOutTextArea.setEditable(false);
        mOutTextArea.setLineWrap(true);         // automatischer Zeilenumbruch verhindert Scrollbalken unten
        mOutTextArea.setWrapStyleWord(true);    // verhindert Zeilenumbruch innerhalb eines Wortes
        mOutTextArea.addListener(this);

        mErrTextArea = new XLogStreamTextArea();
        mErrTextArea.setEditable(false);
        mErrTextArea.setForeground(Color.RED);
        mErrTextArea.setLineWrap(true);         // automatischer Zeilenumbruch verhindert Scrollbalken unten
        mErrTextArea.setWrapStyleWord(true);    // verhindert Zeilenumbruch innerhalb eines Wortes
        mErrTextArea.addListener(this);

        JScrollPane top = new JScrollPane(mOutTextArea);
        JScrollPane buttom = new JScrollPane(mErrTextArea);
        mSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, top, buttom);

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.BOTH;
        c.insets = new Insets(1,1,1,1);
        this.add(mSplitPane,c);
    }

    public void logEvent(XLogStreamTextArea source)
    {
        if ((!mHasOutput) && (source.equals(mOutTextArea)))
        {
            mHasOutput = true;
            if (mSplitPane.getMinimumDividerLocation() >= mSplitPane.getDividerLocation()) mSplitPane.setDividerLocation(0.5);
            return;
        }

        if ((!mHasError) && (source.equals(mErrTextArea)))
        {
            mHasError = true;
            if (mSplitPane.getMaximumDividerLocation() <= mSplitPane.getDividerLocation()) mSplitPane.setDividerLocation(0.5);
            return;
        }
    }

    public XLogStreamTextArea getOutTA()
    {
        return mOutTextArea;
    }

    public XLogStreamTextArea getErrTA()
    {
        return mErrTextArea;
    }
    
    public JSplitPane getSplitPane()
    {
        return mSplitPane;
    }

    public void startLogging(InputStream isOut, InputStream isErr)
    {
        stopLogging();
        mHasOutput = false;
        mHasError = false;
        mOutTextArea.startLogging(isOut);
        mErrTextArea.startLogging(isErr);
    }

    synchronized public void stopLogging()
    {
        mOutTextArea.stopLogging();
        mErrTextArea.stopLogging();
    }
}
